# -*- coding: utf-8 -*-
import numbers
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import attach


def make_leaves(
    pitches,
    durations,
    decrease_durations_monotonically=True,
    forbidden_written_duration=None,
    is_diminution=True,
    metrical_hiearchy=None,
    skips_instead_of_rests=False,
    use_messiaen_style_ties=False,
    use_multimeasure_rests=False,
    ):
    r'''Makes leaves.

    ..  container:: example

        **Example 1.** Integer and string elements in `pitches` result in
        notes:

        ::

            >>> pitches = [2, 4, 'F#5', 'G#5']
            >>> duration = Duration(1, 4)
            >>> leaves = scoretools.make_leaves(pitches, duration)
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                d'4
                e'4
                fs''4
                gs''4
            }

    ..  container:: example

        **Example 2.** Tuple elements in `pitches` result in chords:

        ::

            >>> pitches = [(0, 2, 4), ('F#5', 'G#5', 'A#5')]
            >>> duration = Duration(1, 2)
            >>> leaves = scoretools.make_leaves(pitches, duration)
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                <c' d' e'>2
                <fs'' gs'' as''>2
            }

    ..  container:: example

        **Example 3.** None-valued elements in `pitches` result in rests:

        ::

            >>> pitches = 4 * [None]
            >>> durations = [Duration(1, 4)]
            >>> leaves = scoretools.make_leaves(pitches, durations)
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new RhythmicStaff {
                r4
                r4
                r4
                r4
            }

    ..  container:: example

        **Example 4.** You can mix and match values passed to `pitches`:

        ::

            >>> pitches = [(0, 2, 4), None, 'C#5', 'D#5']
            >>> durations = [Duration(1, 4)]
            >>> leaves = scoretools.make_leaves(pitches, durations)
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                <c' d' e'>4
                r4
                cs''4
                ds''4
            }

    ..  container:: example

        **Example 5.** Read `pitches` cyclically when the length of `pitches`
        is less than the length of `durations`:

        ::

            >>> pitches = ['C5']
            >>> durations = 2 * [Duration(3, 8), Duration(1, 8)]
            >>> leaves = scoretools.make_leaves(pitches, durations)
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c''4.
                c''8
                c''4.
                c''8
            }

    ..  container:: example

        **Example 6.** Read `durations` cyclically when the length of
        `durations` is less than the length of `pitches`:

        ::

            >>> pitches = "c'' d'' e'' f''"
            >>> durations = [Duration(1, 4)]
            >>> leaves = scoretools.make_leaves(pitches, durations)
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c''4
                d''4
                e''4
                f''4
            }

    ..  container:: example

        **Example 7.** Elements in `durations` with non-power-of-two
        denominators result in tuplet-nested leaves:

        ::

            >>> pitches = ['D5']
            >>> durations = [Duration(1, 3), Duration(1, 3), Duration(1, 3)]
            >>> leaves = scoretools.make_leaves(pitches, durations)
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \times 2/3 {
                    d''2
                    d''2
                    d''2
                }
            }

    ..  container:: example

        **Example 8.** Set `decrease_durations_monotonically` to true to
        return nonassignable durations tied from greatest to least:

        ::

            >>> pitches = ['D#5']
            >>> durations = [Duration(13, 16)]
            >>> leaves = scoretools.make_leaves(pitches, durations)
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((13, 16))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 13/16
                ds''2. ~
                ds''16
            }

    ..  container:: example

        **Example 9.** Set `decrease_durations_monotonically` to false
        to return nonassignable durations tied from least to greatest:

        ::

            >>> pitches = ['E5']
            >>> durations = [Duration(13, 16)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     decrease_durations_monotonically=False,
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((13, 16))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 13/16
                e''16 ~
                e''2.
            }

    ..  container:: example

        **Example 10.** Set `forbidden_written_duration` to avoid notes
        greater than or equal to a certain written duration:

        ::

            >>> pitches = "f' g'"
            >>> durations = [Duration(5, 8)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     forbidden_written_duration=Duration(1, 2),
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((5, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 5/4
                f'4 ~
                f'4 ~
                f'8
                g'4 ~
                g'4 ~
                g'8
            }

    ..  container:: example

        **Example 11.** You may set `forbidden_written_duration` and
        `decrease_durations_monotonically` together:

        ::

            >>> pitches = "f' g'"
            >>> durations = [Duration(5, 8)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     forbidden_written_duration=Duration(1, 2),
            ...     decrease_durations_monotonically=False,
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((5, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 5/4
                f'8 ~
                f'4 ~
                f'4
                g'8 ~
                g'4 ~
                g'4
            }

    ..  container:: example

        **Example 12.** Set `is_diminution` to true to produce
        diminished tuplets:

        ::

            >>> pitches = "f'"
            >>> durations = [Duration(5, 14)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     is_diminution=True
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((5, 14))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 5/14
                \tweak edge-height #'(0.7 . 0)
                \times 4/7 {
                    f'2 ~
                    f'8
                }
            }

        This is default behavior.

    ..  container:: example

        **Example 13.** Set `is_diminution` to false to produce
        agumented tuplets:

        ::

            >>> pitches = "f'"
            >>> durations = [Duration(5, 14)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     is_diminution=False
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((5, 14))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 5/14
                \tweak text #tuplet-number::calc-fraction-text
                \tweak edge-height #'(0.7 . 0)
                \times 8/7 {
                    f'4 ~
                    f'16
                }
            }

    ..  container:: example

        **Example 14.** None-valued elements in `pitches` result in
        multimeasure rests when the multimeasure rest keyword is set:

        ::

            >>> pitches = [None]
            >>> durations = [Duration(3, 8), Duration(5, 8)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     use_multimeasure_rests=True,
            ...     )
            >>> leaves
            Selection([MultimeasureRest('R1 * 3/8'), MultimeasureRest('R1 * 5/8')])

        ::

            >>> staff = Staff([
            ...     Measure((3, 8), [leaves[0]]),
            ...     Measure((5, 8), [leaves[1]]),
            ...     ])
            >>> staff.context_name = 'RhythmicStaff'
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 3/8
                    R1 * 3/8
                }
                {
                    \time 5/8
                    R1 * 5/8
                }
            }

    ..  container:: example

        **Example 15.** Uses Messiaen-style ties:

        ::

            >>> pitches = [0]
            >>> durations = [Duration(13, 16)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     use_messiaen_style_ties=True,
            ...     )
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'2.
                c'16 \repeatTie
            }

    ..  container:: example

        **Example 16.** Works with numbered pitch-class:

        ::

            >>> pitches = [pitchtools.NumberedPitchClass(6)]
            >>> durations = [Duration(13, 16)]
            >>> leaves = scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     )
            >>> staff = Staff(leaves)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                fs'2. ~
                fs'16
            }

    ..  container:: example

        **Example 17.** Makes skips instead of rests:

        ::

            >>> pitches = [None]
            >>> durations = [Duration(13, 16)]
            >>> scoretools.make_leaves(
            ...     pitches,
            ...     durations,
            ...     skips_instead_of_rests=True
            ...     )
            Selection([Skip('s2.'), Skip('s16')])

    Returns selection of leaves.
    '''
    from abjad.tools import scoretools
    if isinstance(pitches, str):
        pitches = pitches.split()
    if not isinstance(pitches, list):
        pitches = [pitches]
    if isinstance(durations, (numbers.Number, tuple)):
        durations = [durations]
    nonreduced_fractions = [mathtools.NonreducedFraction(_) for _ in durations]
    size = max(len(nonreduced_fractions), len(pitches))
    nonreduced_fractions = sequencetools.Sequence(nonreduced_fractions)
    nonreduced_fractions = nonreduced_fractions.repeat_to_length(size)
    pitches = sequencetools.Sequence(pitches).repeat_to_length(size)
    Duration = durationtools.Duration
    duration_groups = \
        Duration._group_nonreduced_fractions_by_implied_prolation(
        nonreduced_fractions)
    result = []
    for duration_group in duration_groups:
        # get factors in denominator of duration group other than 1, 2.
        factors = set(mathtools.factors(duration_group[0].denominator))
        factors.discard(1)
        factors.discard(2)
        current_pitches = pitches[0:len(duration_group)]
        pitches = pitches[len(duration_group):]
        if len(factors) == 0:
            for pitch, duration in zip(current_pitches, duration_group):
                leaves = _make_leaf_on_pitch(
                    pitch,
                    duration,
                    decrease_durations_monotonically=decrease_durations_monotonically,
                    forbidden_written_duration=forbidden_written_duration,
                    skips_instead_of_rests=skips_instead_of_rests,
                    use_multimeasure_rests=use_multimeasure_rests,
                    use_messiaen_style_ties=use_messiaen_style_ties,
                    )
                result.extend(leaves)
        else:
            # compute tuplet prolation
            denominator = duration_group[0].denominator
            numerator = mathtools.greatest_power_of_two_less_equal(denominator)
            multiplier = (numerator, denominator)
            ratio = 1 / durationtools.Duration(*multiplier)
            duration_group = [ratio * durationtools.Duration(duration)
                for duration in duration_group]
            # make tuplet leaves
            tuplet_leaves = []
            for pitch, duration in zip(current_pitches, duration_group):
                leaves = _make_leaf_on_pitch(
                    pitch,
                    duration,
                    decrease_durations_monotonically=\
                        decrease_durations_monotonically,
                    skips_instead_of_rests=skips_instead_of_rests,
                    use_multimeasure_rests=use_multimeasure_rests,
                    use_messiaen_style_ties=use_messiaen_style_ties,
                    )
                tuplet_leaves.extend(leaves)
            tuplet = scoretools.Tuplet(multiplier, tuplet_leaves)
            if is_diminution and not tuplet.is_diminution:
                tuplet.toggle_prolation()
            elif not is_diminution and tuplet.is_diminution:
                tuplet.toggle_prolation()
            result.append(tuplet)
    result = selectiontools.Selection(result)
    return result


def _make_leaf_on_pitch(
    pitch,
    duration,
    decrease_durations_monotonically=True,
    forbidden_written_duration=None,
    skips_instead_of_rests=False,
    use_multimeasure_rests=False,
    use_messiaen_style_ties=False,
    ):
    from abjad.tools import scoretools
    note_prototype = (
        numbers.Number,
        str,
        pitchtools.NamedPitch,
        pitchtools.NumberedPitch,
        pitchtools.PitchClass,
        )
    chord_prototype = (tuple, list)
    rest_prototype = (type(None),)
    if isinstance(pitch, note_prototype):
        leaves = scoretools.make_tied_leaf(
            scoretools.Note,
            duration,
            decrease_durations_monotonically=decrease_durations_monotonically,
            forbidden_written_duration=forbidden_written_duration,
            pitches=pitch,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
    elif isinstance(pitch, chord_prototype):
        leaves = scoretools.make_tied_leaf(
            scoretools.Chord,
            duration,
            decrease_durations_monotonically=decrease_durations_monotonically,
            forbidden_written_duration=forbidden_written_duration,
            pitches=pitch,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
    elif isinstance(pitch, rest_prototype) and skips_instead_of_rests:
        leaves = scoretools.make_tied_leaf(
            scoretools.Skip,
            duration,
            decrease_durations_monotonically=decrease_durations_monotonically,
            forbidden_written_duration=forbidden_written_duration,
            pitches=None,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
    elif isinstance(pitch, rest_prototype) and not use_multimeasure_rests:
        leaves = scoretools.make_tied_leaf(
            scoretools.Rest,
            duration,
            decrease_durations_monotonically=decrease_durations_monotonically,
            forbidden_written_duration=forbidden_written_duration,
            pitches=None,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
    elif isinstance(pitch, rest_prototype) and use_multimeasure_rests:
        multimeasure_rest = scoretools.MultimeasureRest((1))
        multiplier = durationtools.Multiplier(duration)
        attach(multiplier, multimeasure_rest)
        leaves = (
            multimeasure_rest,
            )
    else:
        message = 'unknown pitch: {!r}.'
        message = message.format(pitch)
        raise ValueError(message)
    return leaves



# -*- coding: utf-8 -*-
import abc
import collections
import types
from abjad.tools import datastructuretools
from abjad.tools import systemtools
from abjad.tools.datastructuretools import TypedFrozenset


class Set(TypedFrozenset):
    '''Set base class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        if isinstance(items, str):
            items = items.split()
        elif isinstance(items, (
            collections.Iterator,
            types.GeneratorType,
            )):
            items = [item for item in items]
        if item_class is None:
            item_class = self._named_item_class
            if items is not None:
                if isinstance(items, datastructuretools.TypedCollection) and \
                    issubclass(items.item_class, self._parent_item_class):
                    item_class = items.item_class
                elif len(items):
                    if isinstance(items, collections.Set):
                        items = tuple(items)
                    if isinstance(items[0], str):
                        item_class = self._named_item_class
                    elif isinstance(items[0], (int, float)):
                        item_class = self._numbered_item_class
                    elif isinstance(items[0], self._parent_item_class):
                        item_class = type(items[0])
        assert issubclass(item_class, self._parent_item_class)
        TypedFrozenset.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation.

        Returns string.
        '''
        items = self._get_sorted_repr_items()
        items = [str(_) for _ in items]
        return '{{{}}}'.format(', '.join(items))

    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _named_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _numbered_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _parent_item_class(self):
        raise NotImplementedError

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        repr_items = self._get_sorted_repr_items()
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            repr_args_values=[repr_items],
            storage_format_args_values=[repr_items],
            storage_format_kwargs_names=[],
            )

    def _get_sorted_repr_items(self):
        items = sorted(self, key=lambda x: (float(x), str(x)))
        if self.item_class.__name__.startswith('Named'):
            repr_items = [str(x) for x in items]
        elif hasattr(self.item_class, 'pitch_number'):
            repr_items = [x.pitch_number for x in items]
        elif hasattr(self.item_class, 'pitch_class_number'):
            repr_items = [x.pitch_class_number for x in items]
        elif hasattr(self.item_class, '__abs__'):
            repr_items = [abs(x) for x in items]
        else:
            raise ValueError
        return repr_items

    def _sort_self(self):
        return tuple(self)

    ### PUBLIC PROPERTIES ###

    @property
    def cardinality(self):
        r'''Gets cardinality of set.

        Defined equal to length of set.

        Returns nonnegative integer.
        '''
        return len(self)

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes set from `selection`.

        Returns set.
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_skips_01():

    durations = [(1, 2), (1, 3), (1, 4), (1, 5)]
    durations = [Duration(*x) for x in durations]
    staff = Staff(scoretools.make_skips(Duration(1, 4), durations))

    r'''
    \new Staff {
        s4 * 2
        s4 * 4/3
        s4 * 1
        s4 * 4/5
    }
    '''

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            s4 * 2
            s4 * 4/3
            s4 * 1
            s4 * 4/5
        }
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Trill_01():
    r'''Successful trills, showing single leaf overlap.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    trill = spannertools.TrillSpanner()
    attach(trill, target[2:])
    trill = spannertools.TrillSpanner()
    attach(trill, target[:3])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 \startTrillSpan
            c'4
            c'4 \stopTrillSpan \startTrillSpan
            c'4 \stopTrillSpan
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Trill_02():
    r'''Swapped start and stop.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    trill = spannertools.TrillSpanner()
    attach(trill, target[2:])
    trill = spannertools.TrillSpanner()
    attach(trill, target[:3])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 \startTrillSpan
            c'4
            c'4 \stopTrillSpan \startTrillSpan
            c'4 \stopTrillSpan
        }
        '''
        )

    string = r"\relative c' { c \startTrillSpan c c \startTrillSpan \stopTrillSpan c \stopTrillSpan }"

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Trill_03():
    r'''Single leaf.
    '''

    string = r'{ c \startTrillSpan \stopTrillSpan c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Trill_04():
    r'''Unterminated.
    '''

    string = r'{ c \startTrillSpan c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Trill_05():
    r'''Unstarted.
    '''

    string = r'{ c c c c \stopTrillSpan }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Trill_06():
    r'''Nested.
    '''

    string = r'{ c \startTrillSpan c \startTrillSpan c \stopTrillSpan c \stopTrillSpan }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import schemetools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import systemtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import set_
from abjad.tools.pitchtools.PitchClassSegment import PitchClassSegment


class Scale(PitchClassSegment):
    '''A diatonic scale.

    ::

        >>> scale = tonalanalysistools.Scale('c', 'minor')

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_key_signature',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        if len(arguments) == 0:
            key_signature = indicatortools.KeySignature('c', 'major')
        elif len(arguments) == 1 and isinstance(
            arguments[0], indicatortools.KeySignature):
            key_signature = arguments[0]
        elif len(arguments) == 1 and isinstance(arguments[0], Scale):
            key_signature = arguments[0].key_signature
        elif len(arguments) == 2:
            key_signature = indicatortools.KeySignature(*arguments)
        else:
            raise TypeError
        npcs = [key_signature.tonic]
        for mdi in key_signature.mode.named_interval_segment[:-1]:
            named_pitch_class = npcs[-1] + mdi
            npcs.append(named_pitch_class)
        PitchClassSegment.__init__(
            self,
            items=npcs,
            item_class=pitchtools.NamedPitchClass,
            )
        self._key_signature = key_signature

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=[
                str(self.key_signature.tonic),
                self.key_signature.mode.mode_name,
                ],
            )

    def _set_ascending_named_diatonic_pitches_on_logical_ties(
        self, argument):
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        dicg = self.named_interval_class_segment
        length = len(dicg)
        octave_number = 4
        pitch = pitchtools.NamedPitch(self[0], octave_number)
        for i, logical_tie in enumerate(iterate(argument).by_logical_tie()):
            if isinstance(logical_tie[0], scoretools.Note):
                for note in logical_tie:
                    note.written_pitch = pitch
            elif isinstance(logical_tie[0], scoretools.Chord):
                for chord in logical_tie:
                    chord.written_pitches = [pitch]
            else:
                pass
            dic = dicg[i % length]
            ascending_mdi = pitchtools.NamedInterval(dic.quality_string, dic.number)
            pitch += ascending_mdi

    ### PUBLIC METHODS ###

    def create_named_pitch_set_in_pitch_range(self, pitch_range):
        r'''Creates named pitch-set in `pitch_range`.

        Returns pitch-set.
        '''
        if not isinstance(pitch_range, pitchtools.PitchRange):
            pitch_range = pitchtools.PitchRange(
                float(pitchtools.NamedPitch(pitch_range[0])),
                float(pitchtools.NamedPitch(pitch_range[1])))
        low = pitch_range.start_pitch.octave.number
        high = pitch_range.stop_pitch.octave.number
        pitches = []
        octave = low
        while octave <= high:
            for x in self:
                pitch = pitchtools.NamedPitch(x, octave)
                if pitch_range.start_pitch <= pitch and \
                    pitch <= pitch_range.stop_pitch:
                    pitches.append(pitch)
            octave += 1
        return pitchtools.PitchSet(
            items=pitches,
            item_class=pitchtools.NamedPitch,
            )

    @classmethod
    def from_selection(class_, selection, item_class=None, name=None):
        r'''Make scale from `selection`.

        Returns new scale.
        '''
        raise NotImplementedError

    def make_notes(self, n, written_duration=None):
        r'''Makes first `n` notes in ascending diatonic scale
        according to `key_signature`.

        Set `written_duration` equal to `written_duration` or ``1/8``:

        ::

            >>> scale = tonalanalysistools.Scale('c', 'major')
            >>> notes = scale.make_notes(8)
            >>> staff = Staff(notes)

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'8
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8
            }

        ::

            >>> show(staff) # doctest: +SKIP

        Allow nonassignable `written_duration`:

        ::

            >>> notes = scale.make_notes(4, Duration(5, 16))
            >>> staff = Staff(notes)
            >>> time_signature = TimeSignature((5, 4))
            >>> attach(time_signature, staff)

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \time 5/4
                c'4 ~
                c'16
                d'4 ~
                d'16
                e'4 ~
                e'16
                f'4 ~
                f'16
            }

        ::

            >>> show(staff) # doctest: +SKIP

        Returns list of notes.
        '''
        written_duration = written_duration or durationtools.Duration(1, 8)
        result = scoretools.make_notes(n * [0], [written_duration])
        self._set_ascending_named_diatonic_pitches_on_logical_ties(
            result)
        return result

    def make_score(self):
        r'''Make MIDI playback score from scale:

        ::

            >>> scale = tonalanalysistools.Scale('E', 'major')
            >>> score = scale.make_score()
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                tempoWholesPerMinute = #(ly:make-moment 30 1)
            } <<
                \new Staff {
                    \key e \major
                    e'8
                    fs'8
                    gs'8
                    a'8
                    b'8
                    cs''8
                    ds''8
                    e''8
                    ds''8
                    cs''8
                    b'8
                    a'8
                    gs'8
                    fs'8
                    e'4
                }
            >>

        Returns score.
        '''
        ascending_notes = self.make_notes(8, durationtools.Duration(1, 8))
        descending_notes = copy.deepcopy(ascending_notes[:-1])
        descending_notes = list(descending_notes)
        descending_notes.reverse()
        descending_notes = selectiontools.Selection(descending_notes)
        notes = ascending_notes + descending_notes
        notes[-1].written_duration = durationtools.Duration(1, 4)
        staff = scoretools.Staff(notes)
        key_signature = copy.copy(self.key_signature)
        attach(key_signature, staff)
        score = scoretools.Score([staff])
        set_(score).tempo_wholes_per_minute = schemetools.SchemeMoment(30)
        return score

    def named_pitch_class_to_scale_degree(self, *arguments):
        r'''Changes named pitch-class to scale degree.

        Returns scale degree.
        '''
        from abjad.tools import tonalanalysistools
        foreign_pitch_class = pitchtools.NamedPitchClass(*arguments)
        letter = foreign_pitch_class.diatonic_pitch_class_name
        for i, pc in enumerate(self):
            if pc.diatonic_pitch_class_name == letter:
                native_pitch_class = pc
                scale_degree_index = i
                scale_degree_number = scale_degree_index + 1
                break
        native_pitch = pitchtools.NamedPitch(native_pitch_class, 4)
        foreign_pitch = pitchtools.NamedPitch(foreign_pitch_class, 4)
        accidental = foreign_pitch.accidental - native_pitch.accidental
        return tonalanalysistools.ScaleDegree(accidental, scale_degree_number)

    def scale_degree_to_named_pitch_class(self, *arguments):
        r'''Changes scale degree to named pitch-class.

        ::

            >>> scale = tonalanalysistools.Scale('c', 'major')
            >>> scale_degree = tonalanalysistools.ScaleDegree('flat', 5)
            >>> scale.scale_degree_to_named_pitch_class(scale_degree)
            NamedPitchClass('gf')

        ::

            >>> scale_degree = tonalanalysistools.ScaleDegree('flat', 9)
            >>> scale.scale_degree_to_named_pitch_class(scale_degree)
            NamedPitchClass('df')

        Returns named pitch-class.
        '''
        from abjad.tools import tonalanalysistools
        scale_degree = tonalanalysistools.ScaleDegree(*arguments)
        scale_index = (scale_degree.number - 1) % 7
        pitch_class = self[scale_index]
        pitch_class = pitch_class.apply_accidental(scale_degree.accidental)
        return pitch_class

    def voice_scale_degrees_in_open_position(self, scale_degrees):
        r'''Voice `scale_degrees` in open position:

        ::

            >>> scale = tonalanalysistools.Scale('c', 'major')
            >>> scale_degrees = [1, 3, ('flat', 5), 7, ('sharp', 9)]
            >>> segment = scale.voice_scale_degrees_in_open_position(
            ...     scale_degrees)
            >>> segment
            PitchSegment("c' e' gf' b' ds''")

        Return pitch segment.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import tonalanalysistools
        scale_degrees = [tonalanalysistools.ScaleDegree(x)
            for x in scale_degrees]
        pitch_classes = [self.scale_degree_to_named_pitch_class(x)
            for x in scale_degrees]
        pitches = [pitchtools.NamedPitch(pitch_classes[0])]
        for pitch_class in pitch_classes[1:]:
            pitch = pitchtools.NamedPitch(pitch_class)
            while pitch < pitches[-1]:
                pitch += 12
            pitches.append(pitch)
        pitches = pitchtools.PitchSegment(pitches)
        return pitches

    ### PRIVATE PROPERTIES ###

    @property
    def _capital_name(self):
        letter = str(self.key_signature.tonic).title()
        mode = self.key_signature.mode.mode_name.title()
        return '{}{}'.format(letter, mode)

    ### PUBLIC PROPERTIES ###

    @property
    def dominant(self):
        r'''Dominant of scale.

        Return pitch-class.
        '''
        return self[4]

    @property
    def key_signature(self):
        r'''Key signature of scale.

        Returns key signature.
        '''
        return self._key_signature

    @property
    def leading_tone(self):
        r'''Leading tone of scale.

        Returns pitch-class.
        '''
        return self[-1]

    @property
    def mediant(self):
        r'''Mediant of scale.

        Returns pitch-class.
        '''
        return self[2]

    @property
    def named_interval_class_segment(self):
        r'''Named interval class segment of scale.

        Returns interval-class segment.
        '''
        dics = []
        for left, right in sequencetools.Sequence(self).nwise(wrapped=True):
            dic = left - right
            dics.append(dic)
        dicg = pitchtools.IntervalClassSegment(
            items=dics,
            item_class=pitchtools.NamedInversionEquivalentIntervalClass,
            )
        return dicg

    @property
    def subdominant(self):
        r'''Subdominant of scale.

        Returns pitch-class.
        '''
        return self[3]

    @property
    def submediant(self):
        r'''Submediate of scale.

        Returns pitch-class.
        '''
        return self[5]

    @property
    def superdominant(self):
        r'''Superdominant of scale.

        Returns pitch-class.
        '''
        return self[1]

    @property
    def tonic(self):
        r'''Tonic of scale.

        Returns pitch-class.
        '''
        return self[0]



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Parentage_get_first_01():

    staff = Staff("c'8 d'8 e'8 f'8")

    for leaf in staff:
        assert inspect_(leaf).get_parentage().get_first(Staff) is staff

    assert inspect_(staff).get_parentage(include_self=True).get_first(Staff) is staff
    assert inspect_(staff).get_parentage(include_self=False).get_first(Staff) is None



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_RootlessChordClass_from_interval_class_segment_01():

    segment = pitchtools.IntervalClassSegment([
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),])
    class_ = tonalanalysistools.RootlessChordClass
    chord_class = class_.from_interval_class_segment(segment)

    assert chord_class == class_('diminished', 'triad')


def test_tonalanalysistools_RootlessChordClass_from_interval_class_segment_02():

    segment = pitchtools.IntervalClassSegment([
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('major', 3),])
    class_ = tonalanalysistools.RootlessChordClass
    chord_class = class_.from_interval_class_segment(segment)

    assert chord_class == class_('minor', 'triad')


def test_tonalanalysistools_RootlessChordClass_from_interval_class_segment_03():

    segment = pitchtools.IntervalClassSegment([
        pitchtools.NamedInversionEquivalentIntervalClass('major', 3),
        pitchtools.NamedInversionEquivalentIntervalClass('minor', 3),])
    class_ = tonalanalysistools.RootlessChordClass
    chord_class = class_.from_interval_class_segment(segment)

    assert chord_class == class_('major', 'triad')



# -*- coding: utf-8 -*-
import abc
import re
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.pitchtools.Accidental import Accidental


class PitchClass(AbjadValueObject):
    '''Pitch-class base class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _diatonic_pitch_class_name_regex_body = '''
        ([a-g,A-G]) # exactly one lowercase a - g or uppercase A - G
        '''

    _diatonic_pitch_class_name_regex = re.compile(
        '^{}$'.format(_diatonic_pitch_class_name_regex_body),
        re.VERBOSE,
        )

    _diatonic_pitch_class_name_to_diatonic_pitch_class_number = {
        'c': 0,
        'd': 1,
        'e': 2,
        'f': 3,
        'g': 4,
        'a': 5,
        'b': 6,
        }

    _diatonic_pitch_class_name_to_pitch_class_number = {
        'c': 0,
        'd': 2,
        'e': 4,
        'f': 5,
        'g': 7,
        'a': 9,
        'b': 11,
        }

    _diatonic_pitch_class_number_to_diatonic_pitch_class_name = {
        0: 'c',
        1: 'd',
        2: 'e',
        3: 'f',
        4: 'g',
        5: 'a',
        6: 'b',
        }

    _diatonic_pitch_class_number_to_pitch_class_number = {
        0: 0,
        1: 2,
        2: 4,
        3: 5,
        4: 7,
        5: 9,
        6: 11,
        }

    _pitch_class_name_regex_body = '''
        {}          # exactly one diatonic pitch-class name
        {}          # followed by exactly one alphabetic accidental name
        '''.format(
        _diatonic_pitch_class_name_regex_body,
        Accidental._alphabetic_accidental_regex_body,
        )

    _pitch_class_name_regex = re.compile(
        '^{}$'.format(_pitch_class_name_regex_body),
        re.VERBOSE,
        )

    _pitch_class_number_to_diatonic_pitch_class_number = {
        0: 0,
        2: 1,
        4: 2,
        5: 3,
        7: 4,
        9: 5,
        11: 6,
        }

    _pitch_class_number_to_pitch_class_name = {
        0:    'c',
        0.5:  'cqs',
        1:    'cs',
        1.5:  'dqf',
        2:    'd',
        2.5:  'dqs',
        3:    'ef',
        3.5:  'eqf',
        4:    'e',
        4.5:  'eqs',
        5:    'f',
        5.5:  'fqs',
        6:    'fs',
        6.5:  'gqf',
        7:    'g',
        7.5:  'gqs',
        8:    'af',
        8.5:  'aqf',
        9:    'a',
        9.5:  'aqs',
        10:   'bf',
        10.5: 'bqf',
        11:   'b',
        11.5: 'bqs',
        }

    _pitch_class_number_to_pitch_class_name_with_flats = {
        0:    'c',
        0.5:  'dtqf',
        1:    'df',
        1.5:  'dqf',
        2:    'd',
        2.5:  'etqf',
        3:    'ef',
        3.5:  'eqf',
        4:    'e',
        4.5:  'fqf',
        5:    'f',
        5.5:  'gtqf',
        6:    'gf',
        6.5:  'gqf',
        7:    'g',
        7.5:  'atqf',
        8:    'af',
        8.5:  'aqf',
        9:    'a',
        9.5:  'btqf',
        10:   'bf',
        10.5: 'bqf',
        11:   'b',
        11.5: 'cqf',
        }

    _pitch_class_number_to_pitch_class_name_with_sharps = {
        0:    'c',
        0.5:  'cqs',
        1:    'cs',
        1.5:  'ctqs',
        2:    'd',
        2.5:  'dqs',
        3:    'ds',
        3.5:  'dtqs',
        4:    'e',
        4.5:  'eqs',
        5:    'f',
        5.5:  'fqs',
        6:    'fs',
        6.5:  'ftqs',
        7:    'g',
        7.5:  'gqs',
        8:    'gs',
        8.5:  'gtqs',
        9:    'a',
        9.5:  'aqs',
        10:   'as',
        10.5: 'atqs',
        11:   'b',
        11.5: 'bqs',
        }

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats pitch-class.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        if type(self).__name__.startswith('Named'):
            values = [str(self)]
        else:
            values = [
                mathtools.integer_equivalent_number_to_integer(float(self))
                ]
        return systemtools.FormatSpecification(
            client=self,
            coerce_for_equality=True,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            template_names=['pitch_class_name'],
            )

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def accidental(self):
        r'''Accidental of pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def alteration_in_semitones(self):
        r'''Alteration of pitch-class in semitones.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def diatonic_pitch_class_name(self):
        r'''Diatonic pitch-class name corresponding to pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def diatonic_pitch_class_number(self):
        r'''Diatonic pitch-class number corresponding to pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def named_pitch_class(self):
        r'''Named pitch-class corresponding to pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def numbered_pitch_class(self):
        r'''Numbered pitch-class corresponding to pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_class_label(self):
        r'''Pitch-class label of pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_class_name(self):
        r'''Pitch-class name of pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_class_number(self):
        r'''Pitch-class number of pitch-class.
        '''
        raise NotImplementedError

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def apply_accidental(self, accidental=None):
        r'''Applies `accidental` to pitch-class.

        Returns new pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def invert(self, axis=None):
        r'''Inverts pitch-class about `axis`.

        Returns new pitch-class.
        '''
        raise NotImplementedError

    @staticmethod
    def is_diatonic_pitch_class_name(argument):
        '''Is true when `argument` is a diatonic pitch-class name. Otherwise false.

        ::

            >>> pitchtools.PitchClass.is_diatonic_pitch_class_name('c')
            True

        The regex ``^[a-g,A-G]$`` underlies this predicate.

        Returns true or false.
        '''
        if not isinstance(argument, str):
            return False
        return bool(PitchClass._diatonic_pitch_class_name_regex.match(argument))

    @staticmethod
    def is_diatonic_pitch_class_number(argument):
        '''Is true when `argument` is a diatonic pitch-class number. Otherwise false.

        ::

            >>> pitchtools.PitchClass.is_diatonic_pitch_class_number(0)
            True

        ::

            >>> pitchtools.PitchClass.is_diatonic_pitch_class_number(-5)
            False

        The diatonic pitch-class numbers are equal to the set
        ``[0, 1, 2, 3, 4, 5, 6]``.

        Returns true or false.
        '''
        if argument in range(7):
            return True
        return False

    @staticmethod
    def is_pitch_class_name(argument):
        '''Is true when `argument` is a pitch-class name. Otherwise false.

        ::

            >>> pitchtools.PitchClass.is_pitch_class_name('fs')
            True

        The regex ``^([a-g,A-G])(([s]{1,2}|[f]{1,2}|t?q?[fs]|)!?)$`` underlies
        this predicate.

        Returns true or false.
        '''
        if not isinstance(argument, str):
            return False
        return bool(PitchClass._pitch_class_name_regex.match(argument))

    @staticmethod
    def is_pitch_class_number(argument):
        '''True `argument` is a pitch-class number. Otherwise false.

        ::

            >>> pitchtools.PitchClass.is_pitch_class_number(1)
            True

        The pitch-class numbers are equal to the set
        ``[0, 0.5, ..., 11, 11.5]``.

        Returns true or false.
        '''

        return argument in [(n).__truediv__(2) for n in range(24)]

    @abc.abstractmethod
    def multiply(self, n=1):
        r'''Multiplies pitch-class by `n`.

        Returns new pitch-class.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def transpose(self, n=0):
        r'''Transposes pitch-class by index `n`.

        Returns new pitch-class.
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_DuratedComplexBeam___init___01():
    r'''Initialize empty durated complex beam spanner.
    '''

    beam = spannertools.DuratedComplexBeam()
    assert isinstance(beam, spannertools.DuratedComplexBeam)



# -*- coding: utf-8 -*-
import itertools


def yield_all_compositions_of_integer(n):
    r'''Yields all compositions of positive integer `n`.

    ..  container:: example

        ::

            >>> for tuple_ in mathtools.yield_all_compositions_of_integer(5):
            ...     tuple_
            ...
            (5,)
            (4, 1)
            (3, 2)
            (3, 1, 1)
            (2, 3)
            (2, 2, 1)
            (2, 1, 2)
            (2, 1, 1, 1)
            (1, 4)
            (1, 3, 1)
            (1, 2, 2)
            (1, 2, 1, 1)
            (1, 1, 3)
            (1, 1, 2, 1)
            (1, 1, 1, 2)
            (1, 1, 1, 1, 1)

    Lists parts in descending lex order.

    Parts sum to `n`.

    Finds small values of `n` easily.

    Takes around 4 seconds for `n` equal to 17.

    Returns integer tuple generator.
    '''
    from abjad.tools import mathtools
    compositions = []
    x = 0
    string_length = n
    while x < 2 ** (n - 1):
        binary_string = mathtools.integer_to_binary_string(x)
        binary_string = binary_string.zfill(string_length)
        l = [int(c) for c in list(binary_string)]
        partition = []
        g = itertools.groupby(l, lambda x: x)
        for value, group in g:
            partition.append(list(group))
        sublengths = [len(part) for part in partition]
        composition = tuple(sublengths)
        compositions.append(composition)
        x += 1
    for composition in reversed(sorted(compositions)):
        yield composition



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_DuratedComplexBeam_isolated_nib_direction_01():
    r'''Span isolated_nib_direction note when isolated_nib_direction is set to
    true.
    '''

    container = Container("c'8")
    beam = spannertools.DuratedComplexBeam(isolated_nib_direction=True)
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \set stemLeftBeamCount = #1
            \set stemRightBeamCount = #1
            c'8 [ ]
        }
        '''
        )

    assert inspect_(container).is_well_formed()


def test_spannertools_DuratedComplexBeam_isolated_nib_direction_02():
    r'''Do not span isolated_nib_direction note when isolated_nib_direction
    is set to false.
    '''

    container = Container("c'8")
    beam = spannertools.DuratedComplexBeam(isolated_nib_direction=False)
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
        }
        '''
        )

    assert inspect_(container).is_well_formed()


def test_spannertools_DuratedComplexBeam_isolated_nib_direction_03():
    r'''Ignore isolated_nib_direction when spanner spans more than one leaf.
    '''

    container = Container("c'8 d'8")
    beam = spannertools.DuratedComplexBeam(isolated_nib_direction=False)
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #1
            c'8 [
            \set stemLeftBeamCount = #1
            \set stemRightBeamCount = #0
            d'8 ]
        }
        '''
        )

    assert inspect_(container).is_well_formed()


# -*- coding: utf-8 -*-
import re
from docutils.parsers.rst import directives
from sphinx.util.compat import Directive
from sphinx.util.nodes import set_source_info
from docutils.nodes import literal_block


class AbjadDirective(Directive):
    r'''An abjad-book interpreter directive.

    Represents a portion of an interactive session.

    Generates an ``abjad_input_block`` node.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Sphinx Internals'

    ### CLASS VARIABLES ###

    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'allow-exceptions': directives.flag,
        'hide': directives.flag,
        'no-resize': directives.flag,
        'no-stylesheet': directives.flag,
        'no-trim': directives.flag,
        'pages': str,
        'reveal-label': str,
        'strip-prompt': directives.flag,
        'stylesheet': str,
        'text-width': int,
        'with-columns': int,
        'with-thumbnail': directives.flag,
        }

    ### PRIVATE METHODS ###

    @staticmethod
    def _parse_pages_string(pages_string):
        pattern = re.compile(r'(\d+)-(\d+)')
        page_selections = []
        for part in (_.strip() for _ in pages_string.split(',')):
            match = pattern.match(part)
            page_range = None
            if match is not None:
                start, stop = match.groups()
                start = int(start)
                stop = int(stop)
                if start == stop:
                    page_range = (start,)
                elif start < stop:
                    page_range = tuple(range(start, stop + 1))
                else:
                    page_range = tuple(range(start, stop - 1, -1))
            elif part.isdigit():
                page = int(part)
                page_range = (page,)
            else:
                continue
            if page_range:
                page_selections.extend(page_range)
        return tuple(page_selections)

    ### PUBLIC METHODS ###

    def run(self):
        r'''Executes the directive.
        '''
        from abjad.tools import abjadbooktools
        self.assert_has_content()
        code = u'\n'.join(self.content)
        literal = literal_block(code, code)
        literal.line = self.content_offset  # set the content line number
        block = abjadbooktools.abjad_input_block(code, literal)
        # Only set flags if true, for a thinner node repr.
        if 'allow-exceptions' in self.options:
            block['allow-exceptions'] = True
        if 'hide' in self.options:
            block['hide'] = True
        if 'no-resize' in self.options:
            block['no-resize'] = True
        if 'no-stylesheet' in self.options:
            block['no-stylesheet'] = True
        if 'no-trim' in self.options:
            block['no-trim'] = True
        if 'strip-prompt' in self.options:
            block['strip-prompt'] = True
        if 'with-thumbnail' in self.options:
            block['with-thumbnail'] = True
        pages = self.options.get('pages', None)
        if pages is not None:
            block['pages'] = self._parse_pages_string(pages)
        if 'reveal-label' in self.options:
            block['reveal-label'] = self.options.get('reveal-label')
        stylesheet = self.options.get('stylesheet', None)
        if block.get('no-stylesheet'):
            stylesheet = None
        if stylesheet:
            block['stylesheet'] = stylesheet
        text_width = self.options.get('text-width', None)
        if text_width is not None:
            block['text-width'] = text_width
        with_columns = self.options.get('with-columns', None)
        if with_columns is not None:
            block['with-columns'] = with_columns
        set_source_info(self, block)
        return [block]



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_Scale_diatonic_interval_class_segment_01():

    scale = tonalanalysistools.Scale('a', 'major')
    dicg = scale.named_interval_class_segment

    assert str(dicg) == '<+M2, +M2, +m2, +M2, +M2, +M2, +m2>'


def test_tonalanalysistools_Scale_diatonic_interval_class_segment_02():

    scale = tonalanalysistools.Scale('a', 'dorian')
    dicg = scale.named_interval_class_segment

    assert str(dicg) == '<+M2, +m2, +M2, +M2, +M2, +m2, +M2>'



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_StaffChange___repr___01():
    r'''Staff change returns nonempty string repr.
    '''

    staff = Staff([])
    staff.name = 'Flute Staff'
    repr = indicatortools.StaffChange(staff).__repr__()

    assert isinstance(repr, str) and 0 < len(repr)



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_start_offset_01():
    r'''Returns start time of spanner in score.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, container[1:3])
    glissando = spannertools.Glissando()
    attach(glissando, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 \glissando
            d'8 [ \glissando
            e'8 ] \glissando
            f'8
        }
        '''
        )

    assert inspect_(beam).get_timespan().start_offset == Duration(1, 8)
    assert inspect_(glissando).get_timespan().start_offset == Duration(0)


# -*- coding: utf-8 -*-
from __future__ import print_function
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import mutate
Selection = selectiontools.Selection


def _rewrite_meter(
    components,
    meter,
    boundary_depth=None,
    initial_offset=None,
    maximum_dot_count=None,
    rewrite_tuplets=True,
    use_messiaen_style_ties=False,
    ):
    from abjad.tools import metertools
    from abjad.tools import scoretools

    assert isinstance(components, selectiontools.Selection), \
        repr(components)

    if not isinstance(meter, metertools.Meter):
        meter = metertools.Meter(meter)

    boundary_depth = boundary_depth or meter.preferred_boundary_depth

    def recurse(
        boundary_depth=None,
        boundary_offsets=None,
        depth=0,
        logical_tie=None,
        ):
        offsets = metertools.MeterManager.get_offsets_at_depth(
            depth,
            offset_inventory,
            )

        #print('DEPTH:', depth)

        logical_tie_duration = logical_tie._preprolated_duration
        logical_tie_timespan = logical_tie.get_timespan()
        logical_tie_start_offset = logical_tie_timespan.start_offset
        logical_tie_stop_offset = logical_tie_timespan.stop_offset
        logical_tie_starts_in_offsets = logical_tie_start_offset in offsets
        logical_tie_stops_in_offsets = logical_tie_stop_offset in offsets

        if not metertools.MeterManager.is_acceptable_logical_tie(
            logical_tie_duration=logical_tie_duration,
            logical_tie_starts_in_offsets=logical_tie_starts_in_offsets,
            logical_tie_stops_in_offsets=logical_tie_stops_in_offsets,
            maximum_dot_count=maximum_dot_count,
            ):

            #print('UNACCEPTABLE:', logical_tie, logical_tie_start_offset, logical_tie_stop_offset)
            #print('\t', ' '.join([str(x) for x in offsets]))
            split_offset = None
            offsets = metertools.MeterManager.get_offsets_at_depth(
                depth,
                offset_inventory,
                )

            # If the logical tie's start aligns, take the latest possible offset.
            if logical_tie_starts_in_offsets:
                offsets = reversed(offsets)

            for offset in offsets:
                if logical_tie_start_offset < offset < logical_tie_stop_offset:
                    split_offset = offset
                    break

            #print('\tABS:', split_offset)
            if split_offset is not None:
                split_offset -= logical_tie_start_offset
                #print('\tREL:', split_offset)
                #print()
                shards = mutate(logical_tie[:]).split(
                    [split_offset],
                    use_messiaen_style_ties=use_messiaen_style_ties,
                    )
                logical_ties = \
                    [selectiontools.LogicalTie(shard) for shard in shards]
                for logical_tie in logical_ties:
                    recurse(
                        boundary_depth=boundary_depth,
                        boundary_offsets=boundary_offsets,
                        depth=depth,
                        logical_tie=logical_tie,
                        )
            else:
                #print()
                recurse(
                    boundary_depth=boundary_depth,
                    boundary_offsets=boundary_offsets,
                    depth=depth + 1,
                    logical_tie=logical_tie,
                    )

        elif metertools.MeterManager.is_boundary_crossing_logical_tie(
            boundary_depth=boundary_depth,
            boundary_offsets=boundary_offsets,
            logical_tie_start_offset=logical_tie_start_offset,
            logical_tie_stop_offset=logical_tie_stop_offset,
            ):

            #print('BOUNDARY CROSSING', logical_tie, logical_tie_start_offset, logical_tie_stop_offset)
            offsets = boundary_offsets
            if logical_tie_start_offset in boundary_offsets:
                offsets = reversed(boundary_offsets)
            split_offset = None
            for offset in offsets:
                if logical_tie_start_offset < offset < logical_tie_stop_offset:
                    split_offset = offset
                    break
            assert split_offset is not None
            #print('\tABS:', split_offset)
            split_offset -= logical_tie_start_offset
            #print('\tREL:', split_offset)
            #print()
            shards = mutate(logical_tie[:]).split(
                [split_offset],
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            logical_ties = \
                [selectiontools.LogicalTie(shard) for shard in shards]
            for logical_tie in logical_ties:
                recurse(
                    boundary_depth=boundary_depth,
                    boundary_offsets=boundary_offsets,
                    depth=depth,
                    logical_tie=logical_tie,
                    )

        else:
            #print('ACCEPTABLE:', logical_tie, logical_tie_start_offset, logical_tie_stop_offset)
            #print('\t', ' '.join([str(x) for x in offsets]))
            #print()
            logical_tie[:]._fuse()

    # Validate arguments.
    assert Selection._all_are_contiguous_components_in_same_logical_voice(
        components)
    if not isinstance(meter,
        metertools.Meter):
        meter = \
            metertools.Meter(meter)

    #assert sum([x._preprolated_duration for x in components]) == \
    #    meter.preprolated_duration
    if boundary_depth is not None:
        boundary_depth = int(boundary_depth)
    if maximum_dot_count is not None:
        maximum_dot_count = int(maximum_dot_count)
        assert 0 <= maximum_dot_count

    if initial_offset is None:
        initial_offset = durationtools.Offset(0)
    initial_offset = durationtools.Offset(initial_offset)

    first_start_offset = inspect_(components[0]).get_timespan().start_offset
    last_start_offset = inspect_(components[-1]).get_timespan().start_offset
    difference = last_start_offset - first_start_offset + initial_offset
    assert difference < meter.implied_time_signature.duration

    # Build offset inventory, adjusted for initial offset and prolation.
    first_offset = components[0]._get_timespan().start_offset
    first_offset -= initial_offset
    prolation = components[0]._get_parentage(include_self=False).prolation
    offset_inventory = []
    for offsets in meter.depthwise_offset_inventory:
        offsets = [(x * prolation) + first_offset for x in offsets]
        offset_inventory.append(tuple(offsets))

    # Build boundary offset inventory, if applicable.
    if boundary_depth is not None:
        boundary_offsets = offset_inventory[boundary_depth]
    else:
        boundary_offsets = None

    # Cache results of iterator, as we'll be mutating the underlying collection
    iterator = metertools.MeterManager.iterate_rewrite_inputs(components)
    items = tuple(iterator)
    for item in items:
        if isinstance(item, selectiontools.LogicalTie):
            #print('RECURSING:', item)
            recurse(
                boundary_depth=boundary_depth,
                boundary_offsets=boundary_offsets,
                depth=0,
                logical_tie=item,
                )
        elif isinstance(item, scoretools.Tuplet) and rewrite_tuplets == False:
            pass
        else:
            #print('DESCENDING:', item)
            preprolated_duration = sum([x._preprolated_duration for x in item])
            if preprolated_duration.numerator == 1:
                preprolated_duration = mathtools.NonreducedFraction(
                    preprolated_duration)
                preprolated_duration = preprolated_duration.with_denominator(
                    preprolated_duration.denominator * 4)
            sub_metrical_hierarchy = metertools.Meter(preprolated_duration)
            sub_boundary_depth = 1
            if boundary_depth is None:
                sub_boundary_depth = None
            _rewrite_meter(
                item[:],
                sub_metrical_hierarchy,
                boundary_depth=sub_boundary_depth,
                maximum_dot_count=maximum_dot_count,
                )



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__misc__chord_repetition_01():

    target = Container([
        Chord([0, 4, 7], (1, 4)),
        Chord([0, 4, 7], (1, 4)),
        Chord([0, 4, 7], (1, 4)),
        Chord([0, 4, 7], (1, 4)),
    ])

    assert format(target) == stringtools.normalize(
        r'''
        {
            <c' e' g'>4
            <c' e' g'>4
            <c' e' g'>4
            <c' e' g'>4
        }
        '''
        )

    string = r'''{ <c' e' g'> q q q }'''
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__misc__chord_repetition_02():

    target = Staff([
        Chord([0, 4, 7], (1, 8)),
        Chord([0, 4, 7], (1, 8)),
        Chord([0, 4, 7], (1, 4)),
        Chord([0, 4, 7], (3, 16)),
        Chord([0, 4, 7], (1, 16)),
        Chord([0, 4, 7], (1, 4))
    ])

    dynamic = Dynamic('p')
    attach(dynamic, target[0])
    articulation = Articulation('staccatissimo')
    attach(articulation, target[2])
    markup = markuptools.Markup('text', Up)
    attach(markup, target[3])
    articulation = Articulation('staccatissimo')
    attach(articulation, target[-1])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            <c' e' g'>8 \p
            <c' e' g'>8
            <c' e' g'>4 -\staccatissimo
            <c' e' g'>8. ^ \markup { text }
            <c' e' g'>16
            <c' e' g'>4 -\staccatissimo
        }
        '''
        )

    string = r'''\new Staff { <c' e' g'>8\p q q4-| q8.^"text" q16 q4-| }'''
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__misc__chord_repetition_03():

    target = Container([
        Chord([0, 4, 7], (1, 8)),
        Note(12, (1, 8)),
        Chord([0, 4, 7], (1, 8)),
        Note(12, (1, 8)),
        Rest((1, 4)),
        Chord([0, 4, 7], (1, 4)),
    ])

    assert format(target) == stringtools.normalize(
        r'''
        {
            <c' e' g'>8
            c''8
            <c' e' g'>8
            c''8
            r4
            <c' e' g'>4
        }
        '''
        )

    string = r'''{ <c' e' g'>8 c'' q c'' r4 q }'''
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-


def list_all_ide_functions(modules=None):
    r'''Lists all public functions defined in Abjad IDE.

    ::

        >>> all_functions = documentationtools.list_all_ide_functions()  # doctest: +SKIP

    '''
    from abjad.tools import documentationtools
    try:
        return documentationtools.list_all_functions(modules='ide')
    except ImportError:
        return []



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate


def move_full_measure_tuplet_prolation_to_measure_time_signature(argument):
    r'''Moves prolation of full-measure tuplet to time signature of measure.

    Measures usually become non-power-of-two as as result.

    ..  container:: example

        ::

            >>> tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            >>> measure = Measure((2, 8), [tuplet])
            >>> measure.implicit_scaling = True
            >>> scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(measure)

        ..  doctest::

            >>> print(format(measure))
            {
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8
                    d'8
                    e'8
                }
            }

    Returns none.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    for measure in iterate(argument).by_class(scoretools.Measure):
        if len(measure) == 1:
            if isinstance(measure[0], scoretools.Tuplet):
                tuplet = measure[0]
                tuplet_multiplier = tuplet.multiplier
                tuplet_denominator = tuplet_multiplier.denominator
                reduced_denominator = mathtools.remove_powers_of_two(tuplet_denominator)
                time_signature = measure.time_signature
                time_signature_rational = durationtools.Duration(
                    time_signature.numerator, time_signature.denominator)
                numerator = time_signature_rational.numerator * reduced_denominator
                denominator = time_signature_rational.denominator * reduced_denominator
                time_signature = indicatortools.TimeSignature((numerator, denominator))
                detach(indicatortools.TimeSignature, measure)
                attach(time_signature, measure)
                if time_signature.has_non_power_of_two_denominator:
                    measure.implicit_scaling = True
                time_signature_multiplier = \
                    measure.time_signature.implied_prolation
                written_adjustment = tuplet_multiplier.__div__(time_signature_multiplier)
                tuplet._extract()
                measure._scale_contents(written_adjustment)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet_from_selection_01():
    r'''Works with chords.
    '''

    chord = Chord([12, 14, 16], (1, 4))
    named_pitch_class_set_1 = pitchtools.PitchClassSet.from_selection(chord)

    named_pitch_class_set_2 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e')])

    assert named_pitch_class_set_1 == named_pitch_class_set_2


def test_pitchtools_PitchClassSet_from_selection_02():
    r'''Works with notes.
    '''

    note = Note(13, (1, 4))
    named_pitch_class_set_1 = pitchtools.PitchClassSet.from_selection([note])

    named_pitch_class_set_2 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('cs')])

    assert named_pitch_class_set_1 == named_pitch_class_set_2


def test_pitchtools_PitchClassSet_from_selection_03():
    r'''Works with chords.
    '''

    chord = Chord([13, 14, 15], (1, 4))
    pitch_class_set = pitchtools.PitchClassSet.from_selection(chord)
    assert len(pitch_class_set) == 3


def test_pitchtools_PitchClassSet_from_selection_04():
    r'''Works with notes.
    '''

    note = Note(13, (1, 4))
    pitch_class_set = pitchtools.PitchClassSet.from_selection([note])
    assert len(pitch_class_set) == 1



# -*- coding: utf-8 -*-
from __future__ import print_function
import enum
import inspect
import importlib
import os
import re
import shutil
import traceback
import types
from abjad.tools import abctools
from abjad.tools import systemtools


class DocumentationManager(abctools.AbjadObject):
    r'''An API documentation manager.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Documenters'

    api_directory_name = 'api'

    api_title = 'Abjad API'

    lineage_graph_addresses = (
        'abjad',
        'abjad.tools.abjadbooktools',
        'experimental',
        'ide',
        )

    root_package_name = 'abjad'

    source_directory_path_parts = ('docs', 'source')

    tools_packages_package_path = 'abjad.tools'

    ### PRIVATE METHODS ###

    def _build_attribute_section(
        self,
        cls,
        attrs,
        directive,
        title,
        ):
        from abjad.tools import documentationtools
        result = []
        if attrs:
            result.append(documentationtools.ReSTHeading(
                level=3,
                text=title,
                ))
            for attr in attrs:
                options = {
                    #'noindex': True,
                    }
                autodoc = documentationtools.ReSTAutodocDirective(
                    argument='{}.{}.{}'.format(
                        cls.__module__,
                        cls.__name__,
                        attr.name,
                        ),
                    directive=directive,
                    options=options,
                    )
                if cls is attr.defining_class:
                    result.append(autodoc)
                else:
                    container = documentationtools.ReSTDirective(
                        argument='inherited',
                        directive='container',
                        )
                    container.append(autodoc)
                    html_only = documentationtools.ReSTDirective(
                        argument='html',
                        directive='only',
                        )
                    html_only.append(container)
                    result.append(html_only)
        return result

    def _build_attributes_autosummary(
        self,
        cls,
        class_methods,
        data,
        inherited_attributes,
        methods,
        readonly_properties,
        readwrite_properties,
        special_methods,
        static_methods,
        ):
        from abjad.tools import documentationtools
        result = []
        attributes = []
        attributes.extend(readonly_properties)
        attributes.extend(readwrite_properties)
        attributes.extend(methods)
        attributes.extend(class_methods)
        attributes.extend(static_methods)
        attributes.sort(key=lambda x: x.name)
        attributes.extend(special_methods)
        if attributes:
            autosummary = documentationtools.ReSTAutosummaryDirective()
            for attribute in attributes:
                autosummary.append('~{}.{}.{}'.format(
                    cls.__module__,
                    cls.__name__,
                    attribute.name,
                    ))
            html_only = documentationtools.ReSTOnlyDirective(argument='html')
            html_only.append(documentationtools.ReSTHeading(
                level=3,
                text='Attribute summary',
                ))
            html_only.append(autosummary)
            result.append(html_only)
        return result

    def _build_bases_section(self, cls):
        from abjad.tools import documentationtools
        result = []
        result.append(documentationtools.ReSTHeading(
            level=3,
            text='Bases',
            ))
        mro = inspect.getmro(cls)[1:]
        for cls in mro:
            parts = cls.__module__.split('.') + [cls.__name__]
            while 1 < len(parts) and parts[-1] == parts[-2]:
                parts.pop()
            packagesystem_path = '.'.join(parts)
            text = '- :py:class:`{}`'.format(packagesystem_path)
            paragraph = documentationtools.ReSTParagraph(
                text=text,
                wrap=False,
                )
            result.append(paragraph)
        return result

    def _build_enumeration_section(self, cls):
        from abjad.tools import documentationtools
        result = []
        if not issubclass(cls, enum.Enum):
            return result
        items = sorted(cls, key=lambda x: x.name)
        if items:
            result.append(documentationtools.ReSTHeading(
                level=3,
                text='Enumeration Items',
                ))
            for item in items:
                name = item.name
                value = item.value
                line = '- `{}`: {}'.format(name, value)
                paragraph = documentationtools.ReSTParagraph(
                    text=line,
                    wrap=False,
                    )
                result.append(paragraph)
        return result

    def _collect_class_attributes(self, cls):
        ignored_special_methods = (
            '__dict__',
            '__getattribute__',
            '__getnewargs__',
            '__getstate__',
            '__init__',
            '__reduce__',
            '__reduce_ex__',
            '__setstate__',
            '__sizeof__',
            '__subclasshook__',
            'fromkeys',
            'pipe_cloexec',
            )
        class_methods = []
        data = []
        inherited_attributes = []
        methods = []
        readonly_properties = []
        readwrite_properties = []
        special_methods = []
        static_methods = []
        attrs = inspect.classify_class_attrs(cls)
        for attr in attrs:
            if attr.defining_class is object:
                continue
            if attr.defining_class is not cls:
                inherited_attributes.append(attr)
            if attr.kind == 'method':
                if attr.name not in ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        methods.append(attr)
            elif attr.kind == 'class method':
                if attr.name not in ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        class_methods.append(attr)
            elif attr.kind == 'static method':
                if attr.name not in ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        static_methods.append(attr)
            elif attr.kind == 'property' and not attr.name.startswith('_'):
                if attr.object.fset is None:
                    readonly_properties.append(attr)
                else:
                    readwrite_properties.append(attr)
            elif attr.kind == 'data' and not attr.name.startswith('_') \
                and attr.name not in getattr(cls, '__slots__', ()):
                data.append(attr)
        class_methods = tuple(sorted(class_methods))
        data = tuple(sorted(data))
        inherited_attributes = tuple(sorted(inherited_attributes))
        methods = tuple(sorted(methods))
        readonly_properties = tuple(sorted(readonly_properties))
        readwrite_properties = tuple(sorted(readwrite_properties))
        special_methods = tuple(sorted(special_methods))
        static_methods = tuple(sorted(static_methods))
        result = (
            class_methods,
            data,
            inherited_attributes,
            methods,
            readonly_properties,
            readwrite_properties,
            special_methods,
            static_methods,
            )
        return result

    def _ensure_directory(self, path):
        path = os.path.dirname(path)
        if not os.path.exists(path):
            os.makedirs(path)

    def _get_api_directory_path(self, source_directory):
        if self.api_directory_name:
            path = os.path.join(
                source_directory,
                self.api_directory_name,
                )
        else:
            path = source_directory
        return path

    def _get_api_index_file_path(self, source_directory):
        if self.api_directory_name:
            directory_path = os.path.join(
                source_directory,
                self.api_directory_name,
                )
        else:
            directory_path = source_directory
        api_index_path = os.path.join(
            directory_path,
            'index.rst',
            )
        return api_index_path

    def _get_api_index_rst(self, tools_packages):
        from abjad.tools import documentationtools
        document = documentationtools.ReSTDocument()
        heading = documentationtools.ReSTHeading(
            level=2,
            text=self.api_title,
            )
        document.append(heading)
        toc = documentationtools.ReSTTOCDirective(
            options={
                'maxdepth': 3,
                'includehidden': True,
                },
            )
        for tools_package in tools_packages:
            if self.__class__.__name__.startswith('ScoreLibrary'):
                tools_package_parts = tools_package.__name__.split('.')
            else:
                tools_package_parts = tools_package.__name__.split('.')[1:]
            tools_package_path = '/'.join(tools_package_parts)
            text = '{}/index'
            text = text.format(tools_package_path)
            toc_item = documentationtools.ReSTTOCItem(text=text)
            toc.append(toc_item)
        document.append(toc)
        return document

    def _get_class_rst(self, cls):
        import abjad
        module_name, _, class_name = cls.__module__.rpartition('.')
        tools_package_python_path = '.'.join(cls.__module__.split('.')[:-1])
        (
            class_methods,
            data,
            inherited_attributes,
            methods,
            readonly_properties,
            readwrite_properties,
            special_methods,
            static_methods,
            ) = self._collect_class_attributes(cls)
        document = abjad.documentationtools.ReSTDocument()
        module_directive = abjad.documentationtools.ReSTDirective(
            directive='currentmodule',
            argument=tools_package_python_path,
            )
        document.append(module_directive)
        heading = abjad.documentationtools.ReSTHeading(
            level=2,
            text=class_name,
            )
        document.append(heading)
        autoclass_directive = abjad.documentationtools.ReSTAutodocDirective(
            argument=cls.__name__,
            directive='autoclass',
            )
        document.append(autoclass_directive)
        if not self.__class__.__name__.startswith('ScoreLibrary'):
            try:
                lineage_heading = abjad.documentationtools.ReSTHeading(
                    level=3,
                    text='Lineage',
                    )
                document.append(lineage_heading)
                lineage_graph = self._get_lineage_graph(cls)
                lineage_graph.attributes['background'] = 'transparent'
                lineage_graph.attributes['rankdir'] = 'LR'
                graphviz_directive = \
                    abjad.documentationtools.ReSTGraphvizDirective(
                        graph=lineage_graph,
                        )
                graphviz_container = abjad.documentationtools.ReSTDirective(
                    directive='container',
                    argument='graphviz',
                    )
                graphviz_container.append(graphviz_directive)
                document.append(graphviz_container)
            except:
                traceback.print_exc()
        document.extend(self._build_bases_section(cls))
        document.extend(self._build_enumeration_section(cls))
        document.extend(self._build_attributes_autosummary(
            cls,
            class_methods,
            data,
            inherited_attributes,
            methods,
            readonly_properties,
            readwrite_properties,
            special_methods,
            static_methods,
            ))
        document.extend(self._build_attribute_section(
            cls,
            readonly_properties,
            'autoattribute',
            'Read-only properties',
            ))
        document.extend(self._build_attribute_section(
            cls,
            readwrite_properties,
            'autoattribute',
            'Read/write properties',
            ))
        document.extend(self._build_attribute_section(
            cls,
            methods,
            'automethod',
            'Methods',
            ))
        document.extend(self._build_attribute_section(
            cls,
            sorted(class_methods + static_methods, key=lambda x: x.name),
            'automethod',
            'Class & static methods',
            ))
        document.extend(self._build_attribute_section(
            cls,
            special_methods,
            'automethod',
            'Special methods',
            ))
        return document

    def _get_class_summary(self, cls):
        doc = cls.__doc__
        if doc is None:
            doc = ''
        doc = doc.splitlines()
        m = re.search(r"^([A-Z].*?\.)(?:\s|$)", " ".join(doc).strip())
        if m:
            summary = m.group(1).strip()
        elif doc:
            summary = doc[0].strip()
        else:
            summary = ''
        return summary

    def _get_function_rst(self, function):
        import abjad
        document = abjad.documentationtools.ReSTDocument()
        tools_package_python_path = '.'.join(function.__module__.split('.')[:-1])
        module_directive = abjad.documentationtools.ReSTDirective(
            directive='currentmodule',
            argument=tools_package_python_path,
            )
        document.append(module_directive)
        heading = abjad.documentationtools.ReSTHeading(
            level=2,
            text=function.__name__,
            )
        document.append(heading)
        autodoc_directive = abjad.documentationtools.ReSTAutodocDirective(
            argument=function.__name__,
            directive='autofunction',
            )
        document.append(autodoc_directive)
        return document

    def _get_ignored_classes(self):
        from abjad.tools import abjadbooktools
        ignored_classes = set([
            abjadbooktools.abjad_import_block,
            abjadbooktools.abjad_input_block,
            abjadbooktools.abjad_output_block,
            abjadbooktools.abjad_reveal_block,
            abjadbooktools.abjad_thumbnail_block,
            systemtools.TestCase,
            ])
        return ignored_classes

    def _get_lineage_graph(self, cls):
        def get_node_name(original_name):
            parts = original_name.split('.')
            name = [parts[0]]
            for part in parts[1:]:
                if part != name[-1]:
                    name.append(part)
            if name[0] in ('abjad', 'experimental', 'ide'):
                return str('.'.join(name[2:]))
            return str('.'.join(name))
        from abjad.tools import documentationtools
        module_name, _, class_name = cls.__module__.rpartition('.')
        node_name = '{}.{}'.format(cls.__module__, cls.__name__)
        importlib.import_module(module_name)
        lineage_graph_addresses = self._get_lineage_graph_addresses()
        lineage = documentationtools.InheritanceGraph(
            addresses=lineage_graph_addresses,
            lineage_addresses=((module_name, class_name),)
            )
        graph = lineage.__graph__()
        maximum_node_count = 30
        if maximum_node_count < len(graph.leaves):
            lineage = documentationtools.InheritanceGraph(
                addresses=lineage_graph_addresses,
                lineage_addresses=((module_name, class_name),),
                lineage_prune_distance=2,
                )
            graph = lineage.__graph__()
        if maximum_node_count < len(graph.leaves):
            lineage = documentationtools.InheritanceGraph(
                addresses=lineage_graph_addresses,
                lineage_addresses=((module_name, class_name),),
                lineage_prune_distance=1,
                )
            graph = lineage.__graph__()
        if maximum_node_count < len(graph.leaves):
            lineage = documentationtools.InheritanceGraph(
                addresses=((module_name, class_name),),
                )
            graph = lineage.__graph__()
            graph_node = graph[node_name]
            graph_node.attributes['color'] = 'black'
            graph_node.attributes['fontcolor'] = 'white'
            graph_node.attributes['style'] = ('filled', 'rounded')
        graph_node = graph[node_name]
        graph_node.attributes['label'] = \
            '<<B>{}</B>>'.format(graph_node.attributes['label'])
        return graph

    def _get_lineage_graph_addresses(self):
        lineage_graph_addresses = set(self.lineage_graph_addresses)
        lineage_graph_addresses.add(self.root_package_name)
        lineage_graph_addresses = sorted(lineage_graph_addresses)
        return lineage_graph_addresses

    def _get_root_module(self):
        root_module = importlib.import_module(self.root_package_name)
        return root_module

    def _get_source_directory(self):
        if hasattr(self, 'docs_directory'):
            source_directory = os.path.join(self.docs_directory, 'source')
            return source_directory
        root_package = importlib.import_module(self.root_package_name)
        root_package_path = root_package.__path__[0]
        path_parts = [root_package_path]
        path_parts.extend(self.source_directory_path_parts)
        source_directory = os.path.join(*path_parts)
        return source_directory

    def _get_tools_package_contents(self, tools_package):
        classes = []
        functions = []
        for name in dir(tools_package):
            if name.startswith('_'):
                continue
            obj = getattr(tools_package, name)
            if not hasattr(obj, '__module__'):
                if getattr(obj, '__name__', None) == 'abjad':
                    pass
                else:
                    message = 'Warning: no nominative object in {}'
                    message = message.format(obj)
                    print(message)
                continue
            if not obj.__module__.startswith(tools_package.__name__):
                continue
            if isinstance(obj, type):
                classes.append(obj)
            elif isinstance(obj, types.FunctionType):
                functions.append(obj)
        classes.sort(key=lambda x: x.__name__)
        classes = tuple(classes)
        functions.sort(key=lambda x: x.__name__)
        functions = tuple(functions)
        return classes, functions

    def _get_tools_package_graph(self, tools_package):
        from abjad.tools import documentationtools
        lineage_graph_addresses = self._get_lineage_graph_addresses()
        inheritance_graph = documentationtools.InheritanceGraph(
            addresses=lineage_graph_addresses,
            lineage_addresses=[tools_package.__name__]
            )
        lineage_graph = inheritance_graph.__graph__()
        lineage_graph.attributes['bgcolor'] = 'transparent'
        lineage_graph.attributes['dpi'] = 72
        lineage_graph.attributes['rankdir'] = 'LR'
        return lineage_graph

    def _get_tools_package_rst(self, tools_package):
        from abjad.tools import documentationtools
        classes, functions = self._get_tools_package_contents(
            tools_package,
            )
        document = documentationtools.ReSTDocument()
        if self.__class__.__name__.startswith('ScoreLibrary'):
            heading = documentationtools.ReSTHeading(
                level=2,
                text=tools_package.__name__,
                )
        else:
            heading = documentationtools.ReSTHeading(
                level=2,
                text=tools_package.__name__.split('.')[-1],
                )
        document.append(heading)
        automodule_directive = documentationtools.ReSTAutodocDirective(
            argument=tools_package.__name__,
            directive='automodule',
            )
        document.append(automodule_directive)
        ignored_classes = self._get_ignored_classes()
        classes = [_ for _ in classes if _ not in ignored_classes]
        if not self.__class__.__name__.startswith('ScoreLibrary'):
            if classes:
                rule = documentationtools.ReSTHorizontalRule()
                document.append(rule)
                lineage_heading = documentationtools.ReSTHeading(
                    level=3,
                    text='Lineage',
                    )
                document.append(lineage_heading)
                lineage_graph = self._get_tools_package_graph(tools_package)
                graphviz_directive = documentationtools.ReSTGraphvizDirective(
                    graph=lineage_graph,
                    )
                graphviz_container = documentationtools.ReSTDirective(
                    directive='container',
                    argument='graphviz',
                    )
                graphviz_container.append(graphviz_directive)
                document.append(graphviz_container)
        if classes:
            sections = {}
            for cls in classes:
                documentation_section = getattr(
                    cls,
                    '__documentation_section__',
                    None,
                    )
                if documentation_section is None:
                    #if issubclass(cls, enum.Enum):
                    #    documentation_section = 'Enumerations'
                    #elif issubclass(cls, Exception):
                    if issubclass(cls, Exception):
                        documentation_section = 'Errors'
                    else:
                        documentation_section = 'Classes'
                    if inspect.isabstract(cls):
                        documentation_section = 'Abstract Classes'
                if documentation_section not in sections:
                    sections[documentation_section] = []
                sections[documentation_section].append(cls)
            section_names = sorted(sections)
            if 'Main Classes' in sections:
                section_names.remove('Main Classes')
                section_names.insert(0, 'Main Classes')
            if 'Errors' in sections:
                section_names.remove('Errors')
                section_names.append('Errors')
            for section_name in section_names:
                rule = documentationtools.ReSTHorizontalRule()
                document.append(rule)
                heading = documentationtools.ReSTHeading(
                    level=3,
                    text=section_name,
                    )
                document.append(heading)
                toc = documentationtools.ReSTTOCDirective(
                    options={
                        'hidden': True,
                        },
                    )
                for cls in sections[section_name]:
                    class_name = cls.__name__
                    if class_name == 'Index':
                        class_name = '_Index'
                    toc_item = documentationtools.ReSTTOCItem(
                        text=class_name,
                        )
                    toc.append(toc_item)
                document.append(toc)
                autosummary = documentationtools.ReSTAutosummaryDirective(
                    options={
                        'nosignatures': True,
                        },
                    )
                for cls in sections[section_name]:
                    item = documentationtools.ReSTAutosummaryItem(
                        text=cls.__name__,
                        )
                    autosummary.append(item)
                document.append(autosummary)
        if functions:
            if classes:
                rule = documentationtools.ReSTHorizontalRule()
                document.append(rule)
            section_name = 'Functions'
            heading = documentationtools.ReSTHeading(
                level=3,
                text=section_name,
                )
            document.append(heading)
            toc = documentationtools.ReSTTOCDirective(
                options={
                    'hidden': True,
                    },
                )
            for function in functions:
                toc_item = documentationtools.ReSTTOCItem(
                    text=function.__name__,
                    )
                toc.append(toc_item)
            document.append(toc)
            autosummary = documentationtools.ReSTAutosummaryDirective(
                options={
                    'nosignatures': True,
                    },
                )
            for function in functions:
                item = documentationtools.ReSTAutosummaryItem(
                    text=function.__name__,
                    )
                autosummary.append(item)
            document.append(autosummary)
        return document

    def _get_tools_packages(self):
        if hasattr(self, 'packages_to_document'):
            packages_to_document = []
            package_paths = self.packages_to_document.split(',')
            for package_path in package_paths:
                package = importlib.import_module(package_path)
                packages_to_document.append(package)
            return packages_to_document
        root_module = self._get_root_module()
        if os.path.pathsep in self.tools_packages_package_path:
            tools_packages = []
            parts = self.tools_packages_package_path.split(os.path.pathsep)
            for part in parts:
                tools_packages_module = self._get_tools_packages_module(part)
                if getattr(tools_packages_module, '_is_tools_package', None):
                    tools_packages.append(tools_packages_module)
                else:
                    message = 'when passing in a list of paths all should'
                    message += ' be set with _is_tools_package=True.'
                    raise Exception(message)
            return tools_packages
        tools_packages_module = self._get_tools_packages_module()
        if getattr(tools_packages_module, '_is_tools_package', None):
            tools_packages = [tools_packages_module]
            return tools_packages
        tools_packages = []
        for name in dir(tools_packages_module):
            if name.startswith('_'):
                continue
            module = getattr(tools_packages_module, name)
            if not isinstance(module, types.ModuleType):
                continue
            if not module.__name__.startswith(root_module.__name__):
                continue
            tools_packages.append(module)
        tools_packages.sort(key=lambda x: x.__name__)
        tools_packages = tuple(tools_packages)
        return tools_packages

    def _get_tools_packages_module(self, tools_packages_package_path=None):
        if tools_packages_package_path is None:
            tools_packages_package_path = self.tools_packages_package_path
        tools_packages_module = importlib.import_module(
            tools_packages_package_path)
        return tools_packages_module

    def _module_path_to_file_path(self, module_path, source_directory):
        parts = module_path.split('.')
        if not self.__class__.__name__.startswith('ScoreLibrary'):
            parts = parts[1:]
        if parts[-1] == 'Index':
            parts[-1] = '_' + parts[-1] + '.rst'
        else:
            parts[-1] = parts[-1] + '.rst'
        parts.insert(0, self._get_api_directory_path(source_directory))
        path = os.path.join(*parts)
        return path

    def _package_path_to_file_path(self, package_path, source_directory):
        assert isinstance(package_path, str), repr(package_path)
        parts = package_path.split('.')
        if not self.__class__.__name__.startswith('ScoreLibrary'):
            parts = parts[1:]
        parts.append('index.rst')
        parts.insert(0, self._get_api_directory_path(source_directory))
        path = os.path.join(*parts)
        return path

    def _remove_api_directory(self):
        path = self._get_api_directory_path()
        if os.path.exists(path):
            shutil.rmtree(path)

    def _write(self, file_path, string, rewritten_files):
        if not string.endswith('\n'):
            string = '{}\n'.format(string)
        should_write = True
        if os.path.exists(file_path):
            with open(file_path, 'r') as file_pointer:
                old_string = file_pointer.read()
            if old_string == string:
                should_write = False
        if should_write:
            if os.path.exists(file_path):
                print('{}{}'.format(
                    self.prefix_rewrote,
                    os.path.relpath(file_path),
                    ))
            else:
                print('{}{}'.format(
                    self.prefix_wrote,
                    os.path.relpath(file_path),
                    ))
            with open(file_path, 'w') as file_pointer:
                file_pointer.write(string)
        else:
            print('{}{}'.format(
                self.prefix_preserved,
                os.path.relpath(file_path),
                ))
        rewritten_files.add(file_path)

    ### PUBLIC METHODS ###

    def execute(self):
        r'''Executes documentation manager.
        '''
        message = 'Rebuilding documentation source ...'
        print(message)
        source_directory = self._get_source_directory()
        if not os.path.exists(source_directory):
            os.makedirs(source_directory)
        static_html_directory = os.path.join(source_directory, '_static')
        if not os.path.exists(static_html_directory):
            os.makedirs(static_html_directory)
            gitignore_file = os.path.join(static_html_directory, '.gitignore')
            with open(gitignore_file, 'w') as file_pointer:
                file_pointer.write('')
        with systemtools.TemporaryDirectoryChange(
            directory=source_directory,
            verbose=True,
            ):
            rewritten_files = set()
            tools_packages = self._get_tools_packages()
            api_index_rst = self._get_api_index_rst(tools_packages)
            api_index_file_path = self._get_api_index_file_path(
                source_directory)
            self._ensure_directory(api_index_file_path)
            self._write(
                api_index_file_path,
                api_index_rst.rest_format,
                rewritten_files,
                )
            ignored_classes = self._get_ignored_classes()
            for package in tools_packages:
                tools_package_rst = self._get_tools_package_rst(package)
                tools_package_file_path = self._package_path_to_file_path(
                    package.__name__,
                    source_directory,
                    )
                self._ensure_directory(tools_package_file_path)
                self._write(
                    tools_package_file_path,
                    tools_package_rst.rest_format,
                    rewritten_files,
                    )
                classes, functions = \
                    self._get_tools_package_contents(package)
                for cls in classes:
                    file_path = self._module_path_to_file_path(
                        cls.__module__,
                        source_directory,
                        )
                    if cls in ignored_classes:
                        message = '{}{}'
                        message = message.format(
                            self.prefix_ignored,
                            os.path.relpath(file_path),
                            )
                        print(message)
                        continue
                    rst = self._get_class_rst(cls)
                    self._write(file_path, rst.rest_format, rewritten_files)
                for function in functions:
                    file_path = self._module_path_to_file_path(
                        function.__module__,
                        source_directory,
                        )
                    rst = self._get_function_rst(function)
                    self._write(file_path, rst.rest_format, rewritten_files)
            for root, directory_names, file_names in os.walk(
                self._get_api_directory_path(source_directory),
                topdown=False,
                ):
                for file_name in file_names[:]:
                    file_path = os.path.join(root, file_name)
                    if not file_path.endswith('.rst'):
                        continue
                    if file_path not in rewritten_files:
                        file_names.remove(file_name)
                        os.remove(file_path)
                        message = '{}{}'
                        message = message.format(
                            self.prefix_pruned,
                            os.path.relpath(file_path),
                            )
                        print(message)
                if not file_names and not directory_names:
                    shutil.rmtree(root)
                    message = '{}{}'
                    message = message.format(
                        self.prefix_pruned,
                        os.path.relpath(root),
                        )
                    print(message)

    @staticmethod
    def make_readme():
        r'''Creates README.rst file.
        '''
        import abjad
        abjad_path = abjad.__path__[0]
        version = abjad.__version__
        docs_path = os.path.join(abjad_path, 'docs', 'source')
        abstract_path = os.path.join(docs_path, 'abstract.txt')
        badges_path = os.path.join(docs_path, 'badges.txt')
        links_path = os.path.join(docs_path, 'links.txt')
        installation_path = os.path.join(docs_path, 'installation.rst')
        result = 'Abjad {}'.format(version)
        result = ['#' * len(result), result, '#' * len(result)]
        with open(abstract_path, 'r') as file_pointer:
            result.append('')
            result.append(file_pointer.read())
        with open(links_path, 'r') as file_pointer:
            result.append('')
            result.append(file_pointer.read())
        with open(badges_path, 'r') as file_pointer:
            result.append('')
            result.append(file_pointer.read())
        with open(installation_path, 'r') as file_pointer:
            result.append('')
            result.append(file_pointer.read())
        result = '\n'.join(result)
        readme_path = os.path.join(abjad_path, '..', 'README.rst')
        with open(readme_path, 'w') as file_pointer:
            file_pointer.write(result)

    ### PUBLIC PROPERTIES ###

    @property
    def prefix_ignored(self):
        r'''Messaging prefix for ignored files.
        '''
        from sphinx.util.console import lightgray
        return lightgray('IGNORED:   ')

    @property
    def prefix_preserved(self):
        r'''Messaging prefix for preserved files.
        '''
        from sphinx.util.console import darkgray
        return darkgray('PRESERVED: ')

    @property
    def prefix_pruned(self):
        r'''Messaging prefix for pruned files.
        '''
        from sphinx.util.console import red
        return red('PRUNED:    ')

    @property
    def prefix_rewrote(self):
        r'''Messaging prefix for rewritten files.
        '''
        from sphinx.util.console import green
        return green('REWROTE:   ')

    @property
    def prefix_wrote(self):
        r'''Messaging prefix for written files.
        '''
        from sphinx.util.console import yellow
        return yellow('WROTE:     ')



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadValueObject


class DurationSelectorCallback(AbjadValueObject):
    r'''Duration selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_duration',
        '_preprolated',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        duration=durationtools.Duration(1, 4),
        preprolated=None,
        ):
        from abjad.tools import selectortools
        prototype = (
            durationtools.Duration,
            selectortools.DurationInequality,
            )
        assert isinstance(duration, prototype)
        self._duration = duration
        if preprolated is not None:
            preprolated = bool(preprolated)
        self._preprolated = preprolated

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple in which each item is a selection or component.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectortools
        assert isinstance(argument, tuple), repr(argument)
        inequality = self.duration
        if not isinstance(inequality, selectortools.DurationInequality):
            inequality = selectortools.DurationInequality(
                duration=inequality,
                operator_string='==',
                )
        result = []
        for subexpr in argument:
            if not self.preprolated:
                if isinstance(subexpr, scoretools.Component):
                    duration = subexpr._get_duration()
                else:
                    duration = subexpr.get_duration()
            else:
                if isinstance(subexpr, scoretools.Component):
                    subexpr._update_now(offsets=True)
                    duration = subexpr._preprolated_duration
                else:
                    durations = []
                    for x in subexpr:
                        if isinstance(x, scoretools.Component):
                            x._update_now(offsets=True)
                        duration = x._preprolated_duration
                        durations.append(x._preprolated_duration)
                    duration = sum(durations)
            if inequality(duration):
                result.append(subexpr)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def duration(self):
        r'''Gets duration selector callback duration.

        Returns duration or duration inequality.
        '''
        return self._duration

    @property
    def preprolated(self):
        r'''Is true if duration selector callback should be tested against the
        preprolated duration of components in selections. Otherwise false.

        Returns boolean or none.
        '''
        return self._preprolated



# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class MetricAccentKernel(AbjadValueObject):
    r'''A metric accent kernel.

    ..  container:: example

        ::

            >>> hierarchy = metertools.Meter((7, 8))
            >>> kernel = hierarchy.generate_offset_kernel_to_denominator(8)
            >>> kernel
            MetricAccentKernel(
                {
                    Offset(0, 1): Multiplier(3, 14),
                    Offset(1, 8): Multiplier(1, 14),
                    Offset(1, 4): Multiplier(1, 14),
                    Offset(3, 8): Multiplier(1, 7),
                    Offset(1, 2): Multiplier(1, 14),
                    Offset(5, 8): Multiplier(1, 7),
                    Offset(3, 4): Multiplier(1, 14),
                    Offset(7, 8): Multiplier(3, 14),
                    }
                )

    Call the kernel against an expression from which offsets can be counted
    to receive an impulse-response:

    ..  container:: example

        ::

            >>> offsets = [(0, 8), (1, 8), (1, 8), (3, 8)]
            >>> kernel(offsets)
            Multiplier(1, 2)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_kernel',
        '_offsets',
        )

    ### INITIALIZER ###

    def __init__(self, kernel=None):
        kernel = kernel or {}
        assert isinstance(kernel, dict)
        #assert 1 < len(kernel)
        for key, value in kernel.items():
            assert isinstance(key, durationtools.Offset)
            assert isinstance(value, durationtools.Multiplier)
        self._kernel = kernel.copy()
        self._offsets = tuple(sorted(self._kernel))

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        r'''Calls metrical accent kernal on `argument`.

        ::

            >>> upper_staff = Staff("c'8 d'4. e'8 f'4.")
            >>> lower_staff = Staff(r'\clef bass c4 b,4 a,2')
            >>> score = Score([upper_staff, lower_staff])

        ::

            >>> kernel = metertools.MetricAccentKernel.from_meter((4, 4))
            >>> kernel(score)
            Multiplier(10, 33)

        Returns float.
        '''
        offset_count = self.count_offsets(argument)
        response = durationtools.Multiplier(0, 1)
        for offset, count in offset_count.items():
            if offset in self._kernel:
                weight = self._kernel[offset]
                weighted_count = weight * count
                response += weighted_count
        return response

    def __eq__(self, argument):
        r'''Is true when `argument` is a metrical accent kernal with a kernal equal
        to that of this metrical accent kernel. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument):
            if self._kernel == argument._kernel:
                return True
        return False

    def __hash__(self):
        r'''Hashes metric accent kernel.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(MetricAccentKernel, self).__hash__()

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=True,
            storage_format_args_values=[self.kernel],
            storage_format_kwargs_names=[],
            )

    ### PUBLIC METHODS ###

    @staticmethod
    def count_offsets(argument):
        r'''Count offsets in `argument`.

        ..  container:: example

            **Example 1:**

            ::

                >>> upper_staff = Staff("c'8 d'4. e'8 f'4.")
                >>> lower_staff = Staff(r'\clef bass c4 b,4 a,2')
                >>> score = Score([upper_staff, lower_staff])

            ..  doctest::

                >>> print(format(score))
                \new Score <<
                    \new Staff {
                        c'8
                        d'4.
                        e'8
                        f'4.
                    }
                    \new Staff {
                        \clef "bass"
                        c4
                        b,4
                        a,2
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            ::

                >>> MetricAccentKernel = metertools.MetricAccentKernel
                >>> selector = select().by_leaf(flatten=True)
                >>> leaves = selector(score)
                >>> counter = MetricAccentKernel.count_offsets(leaves)
                >>> for offset, count in sorted(counter.items()):
                ...     offset, count
                ...
                (Offset(0, 1), 2)
                (Offset(1, 8), 2)
                (Offset(1, 4), 2)
                (Offset(1, 2), 4)
                (Offset(5, 8), 2)
                (Offset(1, 1), 2)

        ..  container:: example

            **Example 2:**

            ::

                >>> a = timespantools.Timespan(0, 10)
                >>> b = timespantools.Timespan(5, 15)
                >>> c = timespantools.Timespan(15, 20)

            ::

                >>> counter = MetricAccentKernel.count_offsets((a, b, c))
                >>> for offset, count in sorted(counter.items()):
                ...     offset, count
                ...
                (Offset(0, 1), 1)
                (Offset(5, 1), 1)
                (Offset(10, 1), 1)
                (Offset(15, 1), 2)
                (Offset(20, 1), 1)

        Returns counter.
        '''
        from abjad.tools import metertools
        return metertools.OffsetCounter(argument)

    @staticmethod
    def from_meter(meter, denominator=32, normalize=True):
        r'''Create a metric accent kernel from `meter`.

        Returns new metric accent kernel.
        '''
        from abjad.tools import metertools
        if not isinstance(meter, metertools.Meter):
            meter = metertools.Meter(meter)
        return meter.generate_offset_kernel_to_denominator(
            denominator=denominator,
            normalize=normalize,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def duration(self):
        r'''Gets duration.
        '''
        return durationtools.Duration(self._offsets[-1])

    @property
    def kernel(self):
        r'''The kernel datastructure.

        Returns dict.
        '''
        return self._kernel.copy()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_report_modifications_01():

    voice = Voice("c'8 d'8 e'8 f'8")
    comment = indicatortools.LilyPondComment('Example voice', 'before')
    attach(comment, voice)
    override(voice).note_head.color = 'red'
    command = indicatortools.LilyPondCommand("#(set-accidental-style 'forget)")
    attach(command, voice)
    beam = Beam()
    attach(beam, voice[:])
    override(beam).beam.thickness = 3

    assert format(voice) == stringtools.normalize(
        r'''
        % Example voice
        \new Voice \with {
            \override NoteHead.color = #red
        } {
            #(set-accidental-style 'forget)
            \override Beam.thickness = #3
            c'8 [
            d'8
            e'8
            f'8 ]
            \revert Beam.thickness
        }
        '''
        )

    result = inspect_(voice).report_modifications()

    assert format(result) == stringtools.normalize(
        r'''
        % Example voice
        \new Voice \with {
            \override NoteHead.color = #red
        } {
            #(set-accidental-style 'forget)
            %%% 4 components omitted %%%
        }
        '''
        )


def test_agenttools_InspectionAgent_report_modifications_02():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    comment = indicatortools.LilyPondComment('Example tuplet', 'before')
    attach(comment, tuplet)
    override(tuplet).note_head.color = 'red'
    command = indicatortools.LilyPondCommand("#(set-accidental-style 'forget)")
    attach(command, tuplet)
    beam = Beam()
    attach(beam, tuplet[:])
    override(beam).beam.thickness = 3

    assert format(tuplet) == stringtools.normalize(
        r'''
        % Example tuplet
        \override NoteHead.color = #red
        \times 2/3 {
            #(set-accidental-style 'forget)
            \override Beam.thickness = #3
            c'8 [
            d'8
            e'8 ]
            \revert Beam.thickness
        }
        \revert NoteHead.color
        '''
        )

    result = inspect_(tuplet).report_modifications()

    assert format(result) == stringtools.normalize(
        r'''
        % Example tuplet
        \override NoteHead.color = #red
        \times 2/3 {
            #(set-accidental-style 'forget)
            %%% 3 components omitted %%%
        }
        \revert NoteHead.color
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_MultipartBeam_direction_01():

    container = Container("c'8 d'8 r8 e'8 f'8 g'4")
    beam = spannertools.MultipartBeam(direction=Up)
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 ^ [
            d'8 ]
            r8
            e'8 ^ [
            f'8 ]
            g'4
        }
        '''
        )

    detach(beam, container[:])
    beam = spannertools.MultipartBeam(direction=Down)
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 _ [
            d'8 ]
            r8
            e'8 _ [
            f'8 ]
            g'4
        }
        '''
        )


# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.abctools import AbjadObject


class Job(AbjadObject):

    ### INITIALIZER ###

    def __init__(self, number):
        self.number = number

    ### SPECIAL METHODS ###

    def __call__(self):
        self.result = [x for x in mathtools.yield_all_compositions_of_integer(self.number)]


@pytest.mark.skip()
def test_quantizationtools_ParallelJobHandler___call___01():

    jobs = [Job(x) for x in range(1, 11)]
    job_handler = quantizationtools.ParallelJobHandler()
    finished_jobs = job_handler(jobs)


@pytest.mark.skip()
def test_quantizationtools_ParallelJobHandler___call___02():

    job_id = 1
    definition = {
        2: {
            2: {
                2: None
                },
            3: None
            },
        5: None
        }
    search_tree = quantizationtools.UnweightedSearchTree(definition)
    q_event_proxies = [
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent(0, ['A'], index=1), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 5), ['B'], index=2), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 4), ['C'], index=3), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 3), ['D'], index=4), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((2, 5), ['E'], index=5), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 2), ['F'], index=6), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((3, 5), ['G'], index=7), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((2, 3), ['H'], index=8), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((3, 4), ['I'], index=9), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((4, 5), ['J'], index=10), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent(1, ['K'], index=11), 0, 1)
        ]
    job_a = quantizationtools.QuantizationJob(
        job_id, search_tree, q_event_proxies)
    job_b = quantizationtools.QuantizationJob(
        job_id, search_tree, q_event_proxies)

    assert job_a == job_b

    a_jobs = quantizationtools.SerialJobHandler()([job_a])
    b_jobs = quantizationtools.ParallelJobHandler()([job_b])

    assert len(a_jobs) == len(b_jobs)

    a_rtms = sorted([
        q_grid.root_node.rtm_format
        for q_grid in a_jobs[0].q_grids
        ])
    b_rtms = sorted([
        q_grid.root_node.rtm_format
        for q_grid in b_jobs[0].q_grids
        ])

    assert a_rtms == b_rtms

    assert sorted(a_jobs[0].q_grids, key=lambda x: x.root_node.rtm_format) == \
        sorted(b_jobs[0].q_grids, key=lambda x: x.root_node.rtm_format)



# -*- coding: utf-8 -*-


def is_integer_n_tuple(argument, n):
    r'''Is true when `argument` is an integer tuple of length `n`.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_n_tuple((19, 20, 21), 3)
            True

        ::

            >>> mathtools.is_integer_n_tuple((19, 20, 'text'), 3)
            False

    Returns true or false.
    '''

    return (
        isinstance(argument, tuple) and
        len(argument) == n and
        all(isinstance(_, int) for _ in argument)
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Voice_is_nonsemantic_01():

    voice = Voice([])
    assert not voice.is_nonsemantic


def test_scoretools_Voice_is_nonsemantic_02():

    voice = Voice([])
    voice.is_nonsemantic = True

    assert voice.is_nonsemantic


def test_scoretools_Voice_is_nonsemantic_03():

    voice = Voice([])

    assert pytest.raises(TypeError, "voice.is_nonsemantic = 'foo'")



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_selectiontools_Selection__get_crossing_spanners_01():
    r'''Returns unordered set of spanners crossing
    over the begin- or end-bounds of logical-voice-contiguous
    components.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    leaves = select(voice).by_leaf()
    slur = Slur()
    attach(slur, voice[1][:])
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 \startTrillSpan
                d'8
            }
            {
                e'8 (
                f'8 ) \stopTrillSpan
            }
        }
        '''
        )

    spanners = select(voice)._get_crossing_spanners()
    assert spanners == set([])

    spanners = select(leaves)._get_crossing_spanners()
    assert spanners == set([])

    spanners = voice[:1]._get_crossing_spanners()
    assert len(spanners) == 1
    assert trill in spanners

    spanners = select(leaves[:-1])._get_crossing_spanners()
    assert len(spanners) == 2
    assert slur in spanners
    assert trill in spanners


def test_selectiontools_Selection__get_crossing_spanners_02():
    r'''Helper gets spanners that cross in from above.
    '''

    voice = Voice("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves[2:5])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8 [
                f'8
            }
            {
                g'8 ]
                a'8
            }
        }
        '''
        )

    spanners = select(leaves)._get_crossing_spanners()

    assert len(spanners) == 0



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools import schemetools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class TrillSpanner(Spanner):
    r'''Trill spanner.

    ..  container:: example

        Attaches unpitched trill spanner to all notes in staff:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> trill = spannertools.TrillSpanner()
            >>> attach(trill, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'8 \startTrillSpan
                d'8
                e'8
                f'8 \stopTrillSpan
            }

    ..  container:: example

        Attaches pitched trill spanner to all notes in staff:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> trill = spannertools.TrillSpanner(pitch=NamedPitch("cs'"))
            >>> attach(trill, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \pitchedTrill
                c'8 \startTrillSpan cs'
                d'8
                e'8
                f'8 \stopTrillSpan
            }

    ..  container:: example

        Requires at least two leaves:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> trill = spannertools.TrillSpanner()
            >>> attach(trill, staff[:1])
            Traceback (most recent call last):
                ...
            Exception: TrillSpanner() attachment test fails for Selection([Note("c'4")]).

    Formats LilyPond ``\startTrillSpan`` on first leaf in spanner.

    Formats LilyPond ``\stopTrillSpan`` on last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_interval',
        '_is_harmonic',
        '_pitch',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        interval=None,
        is_harmonic=None,
        overrides=None,
        pitch=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        self._interval = interval
        if is_harmonic is not None:
            is_harmonic = bool(is_harmonic)
        self._is_harmonic = is_harmonic
        if pitch is not None:
            pitch = pitchtools.NamedPitch(pitch)
        self._pitch = pitch

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, argument):
        return self._at_least_two_leaves(argument)

    def _copy_keyword_args(self, new):
        self._is_harmonic = self.is_harmonic
        new._pitch = self.pitch

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
            string = r'\startTrillSpan'
            lilypond_format_bundle.right.spanner_starts.append(string)
            if self.pitch is not None or self.interval is not None:
                string = r'\pitchedTrill'
                lilypond_format_bundle.opening.spanners.append(string)
                if self.pitch is not None:
                    string = str(self.pitch)
                    lilypond_format_bundle.right.trill_pitches.append(string)
                elif self.interval is not None:
                    pitch = leaf.written_pitch + self.interval
                    string = str(pitch)
                    lilypond_format_bundle.right.trill_pitches.append(string)
                if self.is_harmonic:
                    string = '(lambda (grob) (grob-interpret-markup grob'
                    string += r' #{ \markup \musicglyph #"noteheads.s0harmonic" #}))'
                    scheme = schemetools.Scheme(string, verbatim=True)
                    override(leaf).trill_pitch_head.stencil = scheme
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
            string = r'\stopTrillSpan'
            lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def interval(self):
        r'''Gets interval of trill.

        ..  container:: example

            Attaches semitone trill:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> trill = spannertools.TrillSpanner(
                ...     interval=pitchtools.NamedInterval('m2'),
                ...     )
                >>> attach(trill, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \pitchedTrill
                    c'8 \startTrillSpan df'
                    d'8
                    e'8
                    f'8 \stopTrillSpan
                }

        ..  container:: example

            Attaches whole tone trill:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> trill = spannertools.TrillSpanner(
                ...     interval=pitchtools.NamedInterval('M2'),
                ...     )
                >>> attach(trill, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \pitchedTrill
                    c'8 \startTrillSpan d'
                    d'8
                    e'8
                    f'8 \stopTrillSpan
                }

        Defaults to none.

        Set to interval or none.

        Returns interval or none.
        '''
        return self._interval

    @property
    def is_harmonic(self):
        r'''Is true when trill pitch note-head should print as a white diamond.
        Otherwise false.

        ..  container:: example

            Attaches harmonic trill:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> trill = spannertools.TrillSpanner(
                ...     is_harmonic=True,
                ...     pitch=NamedPitch("d'"),
                ...     )
                >>> attach(trill, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \once \override TrillPitchHead.stencil = #(lambda (grob) (grob-interpret-markup grob #{ \markup \musicglyph #"noteheads.s0harmonic" #}))
                    \pitchedTrill
                    c'8 \startTrillSpan d'
                    d'8
                    e'8
                    f'8 \stopTrillSpan
                }

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._is_harmonic

    @property
    def pitch(self):
        r'''Gets optional pitch of trill spanner.

        ..  container:: example

            Returns pitch when trill spanner is pitched:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> pitch = NamedPitch('C#4')
                >>> trill = spannertools.TrillSpanner(pitch=pitch)
                >>> attach(trill, staff[:2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \pitchedTrill c'8 \startTrillSpan cs'
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

            ::

                >>> trill.pitch
                NamedPitch("cs'")

        ..  container:: example

            Returns none when trill spanner is unpitched:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> trill = spannertools.TrillSpanner()
                >>> attach(trill, staff[:2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 \startTrillSpan
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

            ::

                >>> trill.pitch is None
                True

        Formats LilyPond ``\pitchedTrill`` command on first leaf in spanner.

        Defaults to none.

        Set to named pitch or none.

        Returns named pitch or none.
        '''
        return self._pitch

    @property
    def written_pitch(self):
        r'''Gets written pitch of trill spanner.

        ..  container:: example

            Returns pitch when trill spanner is pitched:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> pitch = NamedPitch('C#4')
                >>> trill = spannertools.TrillSpanner(pitch=pitch)
                >>> attach(trill, staff[:2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \pitchedTrill c'8 \startTrillSpan cs'
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

            ::

                >>> trill.written_pitch
                NamedPitch("cs'")

        ..  container:: example

            Returns none when trill spanner is unpitched:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> trill = spannertools.TrillSpanner()
                >>> attach(trill, staff[:2])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 \startTrillSpan
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

            ::

                >>> trill.written_pitch is None
                True

        Alias defined equal to `pitch` of trill spanner.

        Defaults to none.

        Property can not be set.

        Returns named pitch or none.
        '''
        return self.pitch



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Container_remove_01():
    r'''Containers remove leaves correctly.
    Leaf detaches from parentage.
    Leaf withdraws from crossing spanners.
    Leaf carries covered spanners forward.
    Leaf returns after removal.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, voice[:])
    beam = Beam()
    attach(beam, voice[1])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 (
            d'8 [ ]
            e'8
            f'8 )
        }
        '''
        )

    note = voice[1]
    voice.remove(note)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 (
            e'8
            f'8 )
        }
        '''
        )

    "Note is now d'8 [ ]"

    assert format(note) == "d'8 [ ]"

    assert inspect_(voice).is_well_formed()
    assert inspect_(note).is_well_formed()


def test_scoretools_Container_remove_02():
    r'''Containers remove nested containers correctly.
    Container detaches from parentage.
    Container withdraws from crossing spanners.
    Container carries covered spanners forward.
    Container returns after removal.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    leaves = select(staff).by_leaf()
    sequential = staff[0]
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    staff.remove(sequential)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                e'8 [
                f'8 ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()

    assert format(sequential) == stringtools.normalize(
        r'''
        {
            c'8
            d'8
        }
        '''
        )

    assert inspect_(sequential).is_well_formed()


def test_scoretools_Container_remove_03():
    r'''Container remove works on identity and not equality.
    '''

    note = Note("c'4")
    container = Container([Note("c'4")])

    assert pytest.raises(Exception, 'container.remove(note)')



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_schemetools_SchemeVectorConstant___setattr___01():
    r'''Scheme vector constants are immutable.
    '''

    scheme_vector_constant = schemetools.SchemeVectorConstant(True, True, False)
    assert pytest.raises(AttributeError, "scheme_vector_constant.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval___le___01():
    r'''Compare two ascending numbered intervals.
    '''

    interval_1 = pitchtools.NumberedInterval(2)
    interval_2 = pitchtools.NumberedInterval(6)

    assert interval_1 <= interval_2
    assert not interval_2 <= interval_1


def test_pitchtools_NumberedInterval___le___02():
    r'''Compare two descending numbered intervals.
    '''

    interval_1 = pitchtools.NumberedInterval(-2)
    interval_2 = pitchtools.NumberedInterval(-6)

    assert interval_1 <= interval_2
    assert not interval_2 <= interval_1


def test_pitchtools_NumberedInterval___le___03():
    r'''Compare two ascending numbered intervals.
    '''

    interval_1 = pitchtools.NumberedInterval(2)
    interval_2 = pitchtools.NumberedInterval(2)

    assert interval_1 <= interval_2
    assert interval_2 <= interval_1


def test_pitchtools_NumberedInterval___le___04():
    r'''Compare two descending numbered intervals.
    '''

    interval_1 = pitchtools.NumberedInterval(-2)
    interval_2 = pitchtools.NumberedInterval(-2)

    assert interval_1 <= interval_2
    assert interval_2 <= interval_1



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.GraceHandler import GraceHandler


class CollapsingGraceHandler(GraceHandler):
    r'''A GraceHandler which collapses pitch information into a single chord
    rather than creating a grace container.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, q_events):
        r'''Calls collapsing grace handler.
        '''
        from abjad.tools import quantizationtools
        pitches = []
        for q_event in q_events:
            if isinstance(q_event, quantizationtools.PitchedQEvent):
                pitches.extend(q_event.pitches)
        return tuple(pitches), None



# -*- coding: utf-8 -*-
from abjad.tools import selectiontools


def make_spacer_skip_measures(time_signatures, implicit_scaling=False):
    r'''Makes measures with full-measure spacer skips from `time_signatures`.

    ..  container:: example

        ::

            >>> measures = scoretools.make_spacer_skip_measures(
            ...     [(1, 8), (5, 16), (5, 16)])
            >>> staff = Staff(measures)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                {
                    \time 1/8
                    s1 * 1/8
                }
                {
                    \time 5/16
                    s1 * 5/16
                }
                {
                    s1 * 5/16
                }
            }

    Returns selection of unincorporated measures.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    # make measures
    measures = []
    for time_signature in time_signatures:
        time_signature = indicatortools.TimeSignature(time_signature)
        measure = scoretools.Measure(
            time_signature,
            implicit_scaling=implicit_scaling,
            )
        measures.append(measure)
    scoretools.fill_measures_with_full_measure_spacer_skips(measures)

    # return measures
    measures = selectiontools.Selection(measures)
    return measures



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class Percussion(Instrument):
    r'''A percussion instrument.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> percussion = instrumenttools.Percussion()
        >>> attach(percussion, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Percussion }
            \set Staff.shortInstrumentName = \markup { Perc. }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    known_percussion = list(sorted(set([
        'agogô',
        'anvil',
        'bass drum',
        'bongo drums',
        'cabasa',
        'cajón',
        'castanets',
        'caxixi',
        'claves',
        'conga drums',
        'cowbell',
        'crotales',
        'cuíca',
        'djembe',
        'finger cymbals',
        'flexatone',
        'frame drum',
        'gong',
        'güiro',
        'hand-held stones',
        'jawbone',
        'maracas',
        'ratchet',
        'rattle',
        'sand blocks',
        'scraped slate',
        'siren',
        'slapstick',
        'slide whistle',
        'snare drum',
        'sponges',
        'suspended cymbal',
        'steel drums',
        'tam-tam',
        'tambourine',
        'temple blocks',
        'thunder machine',
        'thundersheet',
        'toms',
        'tubular bells',
        'triangle',
        'vibraslap',
        'whistle',
        'wind chime',
        'wind machine',
        'wood blocks',
        'wood planks',
        ])))

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='percussion',
        short_instrument_name='perc.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=('percussion',),
        pitch_range=None,
        sounding_pitch_of_written_middle_c=None,
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'percussionist',
            ])

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets percussion's allowable clefs.

        ..  container:: example

            ::

                >>> percussion.allowable_clefs
                ClefList([Clef(name='percussion')])

            ::

                >>> show(percussion.allowable_clefs) # doctest: +SKIP

        Returns clef list.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets percussion's name.

        ..  container:: example

            ::

                >>> percussion.instrument_name
                'percussion'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets percussion's instrument name markup.

        ..  container:: example

            ::

                >>> percussion.instrument_name_markup
                Markup(contents=['Percussion'])

            ::

                >>> show(percussion.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets percussion's range.

        ..  container:: example

            ::

                >>> percussion.pitch_range
                PitchRange(range_string='[A0, C8]')

            ::

                >>> show(percussion.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets percussion's short instrument name.

        ..  container:: example

            ::

                >>> percussion.short_instrument_name
                'perc.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets percussion's short instrument name markup.

        ..  container:: example

            ::

                >>> percussion.short_instrument_name_markup
                Markup(contents=['Perc.'])

            ::

                >>> show(percussion.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of percussion's written middle C.

        ..  container:: example

            ::

                >>> percussion.sounding_pitch_of_written_middle_c
                NamedPitch("c'")

            ::

                >>> show(percussion.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class LaissezVibrer(AbjadValueObject):
    r'''Laissez vibrer.

    ..  container:: example

        ::

            >>> chord = Chord("<c' e' g' c''>4")
            >>> laissez_vibrer = indicatortools.LaissezVibrer()
            >>> attach(laissez_vibrer, chord)
            >>> show(chord) # doctest: +SKIP

        ..  doctest::

            >>> print(format(chord))
            <c' e' g' c''>4 \laissezVibrer

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self):
        self._default_scope = None

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of laissez vibrer indicator.

        ..  container:: example

            **Example 1.** Default:

            ::

                >>> str(indicatortools.LaissezVibrer())
                '\\laissezVibrer'
                    
        Returns string.
        '''
        return r'\laissezVibrer'

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of laissez vibrer indicator.

        ..  container:: example

            **Example 1.** Default:

            ::

                >>> indicator = indicatortools.LaissezVibrer()
                >>> indicator.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_Scale_make_notes_01():
    r'''Allow nonassignable durations.
    '''

    scale = tonalanalysistools.Scale('c', 'major')
    notes = scale.make_notes(2, Duration(5, 16))
    staff = Staff(notes)

    r'''
    \new Staff {
        c'4 ~
        c'16
        d'4 ~
        d'16
    }
    '''

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 ~
            c'16
            d'4 ~
            d'16
        }
        '''
        )



# -*- coding: utf-8 -*-
import textwrap
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


class ImportDirectiveTests(unittest.TestCase):

    def setUp(self):
        self.handler = abjadbooktools.SphinxDocumentHandler

    def test_1(self):
        source = textwrap.dedent('''
        ..  import:: abjad.tools.scoretools.make_notes
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_import_block path="abjad.tools.scoretools.make_notes">
            ''')
        self.assertEqual(result, expected)

    def test_2(self):
        source = textwrap.dedent('''
        ..  import:: abjad.tools.scoretools.make_notes
            :hide:
        ''')
        document = self.handler.parse_rst(source)
        result = stringtools.normalize(document.pformat())
        expected = stringtools.normalize(
            r'''
            <document source="test">
                <abjad_import_block hide="True" path="abjad.tools.scoretools.make_notes">
            ''')
        self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__lilypondfile__LilyPondFile_01():

    string = '{ c } { c } { c } { c }'
    parser = LilyPondParser()
    result = parser(string)
    assert isinstance(result, lilypondfiletools.LilyPondFile)



# -*- coding: utf-8 -*-
import six
from abjad.tools.stringtools.strip_diacritics import strip_diacritics


def to_accent_free_snake_case(string):
    '''Changes `string` to accent-free snake case.

    ..  container:: example

        ::

            >>> stringtools.to_accent_free_snake_case('Déja vu')
            'deja_vu'

    Strips accents from accented characters.

    Changes all punctuation (including spaces) to underscore.

    Sets to lowercase.

    Returns string.
    '''
    assert isinstance(string, six.string_types)
    result = strip_diacritics(string)
    result = result.replace(' ', '_')
    result = result.replace("'", '_')
    result = result.lower()
    return result



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadValueObject


class DurationSpellingSpecifier(AbjadValueObject):
    r'''Duration spelling specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_decrease_durations_monotonically',
        '_forbid_meter_rewriting',
        '_forbidden_written_duration',
        '_rewrite_meter',
        '_spell_metrically',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        decrease_durations_monotonically=True,
        forbid_meter_rewriting=None,
        forbidden_written_duration=None,
        rewrite_meter=None,
        spell_metrically=None,
        ):
        from abjad.tools import rhythmmakertools
        assert isinstance(decrease_durations_monotonically, bool)
        if forbidden_written_duration is not None:
            forbidden_written_duration = durationtools.Duration(
                forbidden_written_duration)
        self._decrease_durations_monotonically = \
            decrease_durations_monotonically
        self._forbidden_written_duration = forbidden_written_duration
        assert isinstance(rewrite_meter, (bool, type(None)))
        self._rewrite_meter = rewrite_meter
        assert (spell_metrically is None or
            isinstance(spell_metrically, bool) or
            spell_metrically == 'unassignable' or
            isinstance(spell_metrically, rhythmmakertools.PartitionTable))
        self._spell_metrically = spell_metrically
        if forbid_meter_rewriting is not None:
            forbid_meter_rewriting = bool(forbid_meter_rewriting)
        self._forbid_meter_rewriting = forbid_meter_rewriting

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats duration spelling specifier.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> f(specifier)
                rhythmmakertools.DurationSpellingSpecifier(
                    decrease_durations_monotonically=True,
                    )

        Returns string.
        '''
        return AbjadValueObject.__format__(
            self,
            format_specification=format_specification,
            )

    def __repr__(self):
        r'''Gets interpreter representation.

        ..  container:: example

            ::

                >>> rhythmmakertools.DurationSpellingSpecifier()
                DurationSpellingSpecifier(decrease_durations_monotonically=True)

        Returns string.
        '''
        return super(DurationSpellingSpecifier, self).__repr__()

    ### PRIVATE METHODS ###

    @staticmethod
    def _rewrite_meter_(
        selections,
        meters,
        reference_meters=None,
        rewrite_tuplets=False,
        use_messiaen_style_ties=False,
        ):
        from abjad.tools import metertools
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import mutate
        meters = [metertools.Meter(_) for _ in meters]
        durations = [durationtools.Duration(_) for _ in meters]
        reference_meters = reference_meters or ()
        selections = DurationSpellingSpecifier._split_at_measure_boundaries(
            selections,
            meters,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
        measures = scoretools.make_spacer_skip_measures(durations)
        staff = scoretools.Staff(measures)
        mutate(staff).replace_measure_contents(selections)
        for measure, meter in zip(staff, meters):
            for reference_meter in reference_meters:
                if str(reference_meter) == str(meter):
                    meter = reference_meter
                    break
            mutate(measure[:]).rewrite_meter(
                meter,
                rewrite_tuplets=rewrite_tuplets,
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
        selections = []
        for measure in staff:
            contents = measure[:]
            for component in contents:
                component._parent = None
            selections.append(contents)
        return selections

    @staticmethod
    def _split_at_measure_boundaries(
        selections,
        meters,
        use_messiaen_style_ties=False,
        ):
        from abjad.tools import metertools
        from abjad.tools import scoretools
        from abjad.tools import sequencetools
        from abjad.tools.topleveltools import inspect_
        from abjad.tools.topleveltools import mutate
        from abjad.tools.topleveltools import select
        meters = [metertools.Meter(_) for _ in meters]
        durations = [durationtools.Duration(_) for _ in meters]
        selections = sequencetools.Sequence(selections).flatten()
        meter_duration = sum(durations)
        music_duration = sum(inspect_(_).get_duration() for _ in selections)
        if not meter_duration == music_duration:
            message = 'Duration of meters is {!s}'
            message += ' but duration of selections is {!s}:'
            message = message.format(meter_duration, music_duration)
            message += '\nmeters: {}.'.format(meters)
            message += '\nmusic: {}.'.format(selections)
            raise Exception(message)
        voice = scoretools.Voice(selections)
        mutate(voice[:]).split(
            durations=durations,
            tie_split_notes=True,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
        #raise Exception(voice)
        #selections = list(voice[:])
        #return selections
        components = mutate(voice).eject_contents()
        component_durations = [inspect_(_).get_duration() for _ in components]
        parts = sequencetools.Sequence(component_durations)
        parts = parts.partition_by_weights(
            weights=durations,
            allow_part_weights=Exact,
            )
        part_lengths = [len(_) for _ in parts]
        parts = sequencetools.Sequence(components).partition_by_counts(
            counts=part_lengths,
            overhang=Exact,
            )
        selections = [select(_) for _ in parts]
        return selections

    ### PUBLIC PROPERTIES ###

    @property
    def decrease_durations_monotonically(self):
        r'''Is true when all durations should be spelled as a tied series of
        monotonically decreasing values. Otherwise false.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.decrease_durations_monotonically
                True

        Defaults to true.

        Returns true or false.
        '''
        return self._decrease_durations_monotonically

    @property
    def forbid_meter_rewriting(self):
        r'''Is true when meter rewriting is forbidden.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.forbid_meter_rewriting is None
                True

        Defaults to none.

        Returns boolean or none.
        '''
        return self._forbid_meter_rewriting

    @property
    def forbidden_written_duration(self):
        r'''Gets forbidden written duration.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.forbidden_written_duration is None
                True

        Defaults to none.

        Returns duration or none.
        '''
        return self._forbidden_written_duration

    @property
    def rewrite_meter(self):
        r'''Is true when all output divisions should rewrite meter.
        Otherwise false.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.rewrite_meter is None
                True

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._rewrite_meter

    @property
    def spell_metrically(self):
        r'''Is true when durations should spell according to approximate common
        practice understandings of meter. Otherwise false.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.DurationSpellingSpecifier()
                >>> specifier.spell_metrically is None
                True

        Spells unassignable durations like ``5/16`` and ``9/4`` metrically when
        set to ``'unassignable'``. Leaves other durations unchanged.

        Defaults to none.

        Returns boolean, ``'unassignable'`` or none..
        '''
        return self._spell_metrically



# -*- coding: utf-8 -*-
import abjad
from abjad.tools import lilypondfiletools


def make_part_lilypond_file():
    r'''Makes Pärt LilyPond file.
    '''
    score_template = abjad.demos.part.PartCantusScoreTemplate()
    score = score_template()
    abjad.demos.part.add_bell_music_to_score(score)
    abjad.demos.part.add_string_music_to_score(score)
    abjad.demos.part.apply_bowing_marks(score)
    abjad.demos.part.apply_dynamics(score)
    abjad.demos.part.apply_expressive_marks(score)
    abjad.demos.part.apply_page_breaks(score)
    abjad.demos.part.apply_rehearsal_marks(score)
    abjad.demos.part.apply_final_bar_lines(score)
    abjad.demos.part.configure_score(score)
    lilypond_file = lilypondfiletools.LilyPondFile.new(score)
    abjad.demos.part.configure_lilypond_file(lilypond_file)
    return lilypond_file


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_schemetools_SchemeAssociativeList___setattr___01():
    r'''Scheme associative lists are immutable.
    '''

    scheme_alist = schemetools.SchemeAssociativeList(
        ('space', 2), ('padding', 0.5))
    assert pytest.raises(AttributeError, "scheme_alist.foo = 'bar'")
